--------------------------------------------------------------------------
-- Archaeologist.lua 
--------------------------------------------------------------------------
--[[
Archaeologist 
	Unearthing Health and Mana values at a unit frame near you.

By: AnduinLothar    <KarlKFI@ultimateuiui.org>

Change Log:
v0.1a
-Adds Text to Unit Frame Status Bars
-Adds Dead/Offline/Ghost text to all unit frames
-Changes health bar color as it decreases
-Status bar prefix is optional
-All status text can be shown as percent or real value (except target HP which is percent from server)

v0.2a
-Fixed Slash commands not working for text.
-Sepperated Player HP/MP/XP text
-Made target percent an option so you can see true values when you target self or party member

v0.3a
-Optimized Slash and UltimateUI Reg Code.
-Fixed non-UltimateUI saved variables not being saved (also nil pointer bug)
-Fixed text relocation so that the HP and MP text are now slightly sepperated
-Removed partial frame replacement and replaced it with onLoad status bar initializing

v0.4a
-Fixed a hp prefix reshowing bug
-Added Pet Dead Text
-Fixed Pet MP/HP show issue

v0.5a
-TargetFrame healthbars now always display as percent unless max hp is difforent thant 100 (party, player, pet)

v0.6a
-Added 12 Buffs and 12 Debuffs for all party members and pet.
-Moved Pet Happiness to accomidate pet debuffs
-Options to Show or Hide buffs or debuffs (reverts to normal onmouseover behavior when off)
-Show anywhere between 0 and 12 Buffs or Debuffs
-Cleaned up slash command printouts
-Syncronized UltimateUI with Slash Commands. Use both if you're thus inspired.

v0.7a
-Added Alternate Debuff Location.
-Added buff tooltips
-Changed Slash Command syntax
-Added Alternate HPMP text location that aligns to the outside of the bar frame.

v1.0
-Live in UltimateUI!
-Spelling Errors Corrected

v1.1
-Added German Localization
-Sorting of ultimateui options.
-Auto Pet Happiness Alt Location Management

v1.2
-Added French Localization
-Fixed ArchaeologistVars = nil error
-Text is now resizable from 6-20.
Note: due to a blizzard text resizing bug the text will not propperly smooth/rescale once changed.
To correct it go to windowed mode and back to fullscreen.  I have notified Blizzard of the bug.
-Made Party/Pet Buff borders green. Debuffs are still red.

v1.3
-Added 8 Buffs and 8 Debuffs for target frame.
-Target Buffs slightly rearanged. They now extend from right to left so that the 3 extra buffs don't
get covered by the portrait and elite graphics.
-Increased Pet and Party Max buffs/debuffs to 14

v1.4
-Fixed the bug that hid status text while moused over.
-Fixed the bug that hid status text while the character frame was open.

v1.41
-Implimented Smooth Text Resizing Work-around

v1.42
-Fixed a bug with loading default values from the new text sliders w/o ultimateui.

v1.43
-Fixed a bigger bug with values not saving with slash commands and ultimateui.

v1.5
-Made Target Buffs on top if targeting an ally and Debuffs on top if targeting an enemy.
-Added an alternate option for target buffs to display from right to left. Default is left to right.

	$Id: Archaeologist.lua 1398 2005-04-27 10:23:03Z karlkfi $
	$Rev: 1398 $
	$LastChangedBy: karlkfi $
	$Date: 2005-04-27 05:23:03 -0500 (Wed, 27 Apr 2005) $

]]--

-- <= == == == == == == == == == == == == =>
-- => Global Variables
-- <= == == == == == == == == == == == == =>

Archaeologist_TextStringPercentStatusBars = { };

ArchaeologistStatusBars = { };

function Archaeologist_DefineStatusBars()

	ArchaeologistStatusBars.player  = { frame = PlayerFrame, statusText = PlayerStatusText };
	ArchaeologistStatusBars.party1  = { frame = PartyMemberFrame1, statusText = PartyMemberFrame1.statusText };
	ArchaeologistStatusBars.party2  = { frame = PartyMemberFrame2, statusText = PartyMemberFrame2.statusText };
	ArchaeologistStatusBars.party3  = { frame = PartyMemberFrame3, statusText = PartyMemberFrame3.statusText };
	ArchaeologistStatusBars.party4  = { frame = PartyMemberFrame4, statusText = PartyMemberFrame4.statusText };
	ArchaeologistStatusBars.pet		= { frame = PetFrame, statusText = PetStatusText };
	ArchaeologistStatusBars.target  = { frame = TargetFrame, statusText = TargetDeadText };

end

Archaeologist_TargetBuffsAreOnTop = false;

-- <= == == == == == == == == == == == == =>
-- => Variable Sync Tables
-- <= == == == == == == == == == == == == =>

ArchaeologistVars = { };
ArchaeologistVarData = { };

function Archaeologist_DefineVarData()
	ArchaeologistVarData = {

		PLAYERHP = { ultimateui = "UUI_ARCHAEOLOGIST_PLAYERHP", default = 0, func = Archaeologist_TurnOnPlayerHP },
		PLAYERMP = { ultimateui = "UUI_ARCHAEOLOGIST_PLAYERMP", default = 0, func = Archaeologist_TurnOnPlayerMP },
		PLAYERXP = { ultimateui = "UUI_ARCHAEOLOGIST_PLAYERXP", default = 0, func = Archaeologist_TurnOnPlayerXP },
		PLAYERHPP = { ultimateui = "UUI_ARCHAEOLOGIST_PLAYERHPP", default = 0, func = Archaeologist_TurnOnPlayerHPPercent },
		PLAYERMPP = { ultimateui = "UUI_ARCHAEOLOGIST_PLAYERMPP", default = 0, func = Archaeologist_TurnOnPlayerMPPercent },
		PLAYERXPP = { ultimateui = "UUI_ARCHAEOLOGIST_PLAYERXPP", default = 0, func = Archaeologist_TurnOnPlayerXPPercent },
		PLAYERHPJUSTVALUE = { ultimateui = "UUI_ARCHAEOLOGIST_PLAYERHPJUSTVALUE", default = 0, func = Archaeologist_TurnOffPlayerHPPrefix },
		PLAYERMPJUSTVALUE = { ultimateui = "UUI_ARCHAEOLOGIST_PLAYERMPJUSTVALUE", default = 0, func = Archaeologist_TurnOffPlayerMPPrefix },
		PLAYERXPJUSTVALUE = { ultimateui = "UUI_ARCHAEOLOGIST_PLAYERXPJUSTVALUE", default = 0, func = Archaeologist_TurnOffPlayerXPPrefix },
		
		PARTYHP = { ultimateui = "UUI_ARCHAEOLOGIST_PARTYHP", default = 0, func = Archaeologist_TurnOnPartyHP },
		PARTYMP = { ultimateui = "UUI_ARCHAEOLOGIST_PARTYMP", default = 0, func = Archaeologist_TurnOnPartyMP },
		PARTYHPP = { ultimateui = "UUI_ARCHAEOLOGIST_PARTYHPP", default = 0, func = Archaeologist_TurnOnPartyHPPercent },
		PARTYMPP = { ultimateui = "UUI_ARCHAEOLOGIST_PARTYMPP", default = 0, func = Archaeologist_TurnOnPartyMPPercent },
		PARTYHPJUSTVALUE = { ultimateui = "UUI_ARCHAEOLOGIST_PARTYHPJUSTVALUE", default = 1, func = Archaeologist_TurnOffPartyHPPrefix },
		PARTYMPJUSTVALUE = { ultimateui = "UUI_ARCHAEOLOGIST_PARTYMPJUSTVALUE", default = 0, func = Archaeologist_TurnOffPartyMPPrefix },
		
		PETHP = { ultimateui = "UUI_ARCHAEOLOGIST_PETHP", default = 0, func = Archaeologist_TurnOnPetHP },
		PETMP = { ultimateui = "UUI_ARCHAEOLOGIST_PETMP", default = 0, func = Archaeologist_TurnOnPetMP },
		PETHPP = { ultimateui = "UUI_ARCHAEOLOGIST_PETHPP", default = 0, func = Archaeologist_TurnOnPetHPPercent },
		PETMPP = { ultimateui = "UUI_ARCHAEOLOGIST_PETMPP", default = 0, func = Archaeologist_TurnOnPetMPPercent },
		PETHPJUSTVALUE = { ultimateui = "UUI_ARCHAEOLOGIST_PETHPJUSTVALUE", default = 1, func = Archaeologist_TurnOffPetHPPrefix },
		PETMPJUSTVALUE = { ultimateui = "UUI_ARCHAEOLOGIST_PETMPJUSTVALUE", default = 0, func = Archaeologist_TurnOffPetMPPrefix },
		
		TARGETHP = { ultimateui = "UUI_ARCHAEOLOGIST_TARGETHP", default = 0, func = Archaeologist_TurnOnTargetHP },
		TARGETMP = { ultimateui = "UUI_ARCHAEOLOGIST_TARGETMP", default = 0, func = Archaeologist_TurnOnTargetMP },
		TARGETHPP = { ultimateui = "UUI_ARCHAEOLOGIST_TARGETHPP", default = 0, func = Archaeologist_TurnOnTargetHPPercent },
		TARGETMPP = { ultimateui = "UUI_ARCHAEOLOGIST_TARGETMPP", default = 0, func = Archaeologist_TurnOnTargetMPPercent },
		TARGETHPJUSTVALUE = { ultimateui = "UUI_ARCHAEOLOGIST_TARGETHPJUSTVALUE", default = 1, func = Archaeologist_TurnOffTargetHPPrefix },
		TARGETMPJUSTVALUE = { ultimateui = "UUI_ARCHAEOLOGIST_TARGETMPJUSTVALUE", default = 0, func = Archaeologist_TurnOffTargetMPPrefix },
		
		HPCOLOR = { ultimateui = "UUI_ARCHAEOLOGIST_HPCOLOR", default = 0, func = Archaeologist_TurnOnHPColorMod },
		
		PBUFFS = { ultimateui = "UUI_ARCHAEOLOGIST_PBUFFS", default = 0, func = Archaeologist_TurnOffPartyBuffs },
		PBUFFNUM = { ultimateui = "UUI_ARCHAEOLOGIST_PBUFFNUM", default = 14, min = 0, max = 14, func = Archaeologist_SetPartyBuffs },
		
		PDEBUFFS = { ultimateui = "UUI_ARCHAEOLOGIST_PDEBUFFS", default = 0, func = Archaeologist_TurnOffPartyDebuffs },
		PDEBUFFNUM = { ultimateui = "UUI_ARCHAEOLOGIST_PDEBUFFNUM", default = 14, min = 0, max = 14, func = Archaeologist_SetPartyDebuffs },
		
		PTBUFFS = { ultimateui = "UUI_ARCHAEOLOGIST_PTBUFFS", default = 0, func = Archaeologist_TurnOffPetBuffs },
		PTBUFFNUM = { ultimateui = "UUI_ARCHAEOLOGIST_PTBUFFNUM", default = 14, min = 0, max = 14, func = Archaeologist_SetPetBuffs },
		
		PTDEBUFFS = { ultimateui = "UUI_ARCHAEOLOGIST_PTDEBUFFS", default = 0, func = Archaeologist_TurnOffPetDebuffs },
		PTDEBUFFNUM = { ultimateui = "UUI_ARCHAEOLOGIST_PTDEBUFFNUM", default = 4, min = 0, max = 14, func = Archaeologist_SetPetDebuffs },
		
		TBUFFS = { ultimateui = "UUI_ARCHAEOLOGIST_TBUFFS", default = 0, func = Archaeologist_TurnOffTargetBuffs },
		TBUFFNUM = { ultimateui = "UUI_ARCHAEOLOGIST_TBUFFNUM", default = 8, min = 0, max = 8, func = Archaeologist_SetTargetBuffs },
		
		TDEBUFFS = { ultimateui = "UUI_ARCHAEOLOGIST_TDEBUFFS", default = 0, func = Archaeologist_TurnOffTargetDebuffs },
		TDEBUFFNUM = { ultimateui = "UUI_ARCHAEOLOGIST_TDEBUFFNUM", default = 8, min = 0, max = 8, func = Archaeologist_SetTargetDebuffs },
		
		DEBUFFALT = { ultimateui = "UUI_ARCHAEOLOGIST_DEBUFFALT", default = 0, func = Archaeologist_SetAltDebuffLocation },
		TBUFFALT = { ultimateui = "UUI_ARCHAEOLOGIST_TBUFFALT", default = 0, func = Archaeologist_TargetBuffs_SetAltHorizontalAlignment },
		HPMPALT = { ultimateui = "UUI_ARCHAEOLOGIST_HPMPALT", default = 0, func = Archaeologist_SetAltHPMPLocation },
		HPMPLARGESIZE = { ultimateui = "UUI_ARCHAEOLOGIST_HPMPLARGESIZE", default = 14, min = 6, max = 20, func = Archaeologist_SetHPMPLargeTextSize },
		HPMPSMALLSIZE = { ultimateui = "UUI_ARCHAEOLOGIST_HPMPSMALLSIZE", default = 14, min = 6, max = 20, func = Archaeologist_SetHPMPSmallTextSize }

	};
end

-- <= == == == == == == == == == == == == =>
-- => XML Function Calls
-- <= == == == == == == == == == == == == =>

local SavedHealthBar_OnValueChanged = nil;

function Archaeologist_OnLoad()
	Sea.util.hook( "TargetFrame_CheckDead", "Archaeologist_TargetCheckDead", "replace" );
	Sea.util.hook( "ShowTextStatusBarText", "Archaeologist_ShowTextStatusBarText", "replace" );
	Sea.util.hook( "HideTextStatusBarText", "Archaeologist_HideTextStatusBarText", "replace" );
	Sea.util.hook( "TextStatusBar_UpdateTextString", "Archaeologist_TextStatusBar_UpdateTextString", "replace" );
	Sea.util.hook( "CharacterFrame_OnShow", "Archaeologist_CharacterFrame_OnShow", "replace" );
	Sea.util.hook( "CharacterFrame_OnHide", "Archaeologist_CharacterFrame_OnHide", "replace" );
	Sea.util.hook( "UnitFrame_UpdateManaType", "Archaeologist_UnitFrame_UpdateManaType", "replace" );
	
	Sea.util.hook( "PartyMemberFrame_RefreshBuffs", "Archaeologist_PartyMemberFrame_RefreshBuffs", "replace" );
	Sea.util.hook( "PetFrame_RefreshBuffs", "Archaeologist_PetFrame_RefreshBuffs", "replace" );
	Sea.util.hook( "PartyMemberBuffTooltip_Update", "Archaeologist_PartyMemberBuffs_Update", "replace" );
	Sea.util.hook( "ShowPartyFrame", "Archaeologist_UpdatePartyMemberBuffs", "after" );
	Sea.util.hook( "TargetDebuffButton_Update", "Archaeologist_TargetDebuffButton_Update", "replace" );
	
	--HealthBar_OnValueChanged manual hook to modify input
	if (HealthBar_OnValueChanged ~= SavedHealthBar_OnValueChanged) then
		SavedHealthBar_OnValueChanged = HealthBar_OnValueChanged;
		HealthBar_OnValueChanged = Archaeologist_HealthBar_OnValueChanged;
	end
	
	--Sea.util.hook( "MouseIsOver", "Archaeologist_MouseIsOver", "replace" );
	
	this:RegisterEvent("VARIABLES_LOADED");
	this:RegisterEvent("UNIT_HEALTH");
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("PARTY_MEMBERS_CHANGED");
	this:RegisterEvent("PARTY_MEMBER_DISABLE");
	this:RegisterEvent("PARTY_MEMBER_ENABLE");
	this:RegisterEvent("UNIT_AURA");
	
	Archaeologist_DefineVarData();
	Archaeologist_InitializeAddedStatusBarTexts();
	Archaeologist_DefineStatusBars();
	Archaeologist_RegisterForSlashCommands();
	Archaeologist_VarSync_SavedToVars(); --set all to default since nothing has loaded yet
end

function Archaeologist_OnEvent(event)
	if (event == "VARIABLES_LOADED") then
		
		if (not ArchaeologistVars) then
			ArchaeologistVars = { };
		end
		
		if (UltimateUI_RegisterConfiguration) then
			Archaeologist_VarSync_UltimateUIToVars();
			Archaeologist_RegisterForUltimateUI();
		else
			Archaeologist_VarSync_SavedToVars()
			Archaeologist_VarSync_VarsToLive();
			RegisterForSave("ArchaeologistVars");
		end
		
		Archaeologist_PlayerCheckDead();
		Archaeologist_UpdatePartyMembersDead();
		Archaeologist_PetCheckDead();
		Archaeologist_HideOrigTargetBuffs();
	
	elseif ( event == "PLAYER_ENTERING_WORLD" ) then
		--Archaeologist_PlayerCheckDead();
		--Archaeologist_UpdatePartyMembersDead();
		--Archaeologist_PetCheckDead();
		Archaeologist_UpdatePartyMemberBuffs();
	
	elseif ( event == "UNIT_HEALTH" ) then
		if (arg1 == "player") then
			Archaeologist_PlayerCheckDead();
		elseif (arg1 == "target") then
			--called by hook
			--Archaeologist_TargetCheckDead();
		elseif (arg1 == "pet") then
			Archaeologist_PetCheckDead();
		else
			local partyIndex = Archaeologist_PartyIndexFromUnit(arg1);
			Archaeologist_PartyCheckDead(partyIndex);
		end
		
	elseif (event == "PARTY_MEMBERS_CHANGED") then
		Archaeologist_UpdatePartyMembersDead();
		--Sea.io.print(event,arg1,arg2,arg3,arg4,arg5);
		
	elseif (event == "PARTY_MEMBER_ENABLE") then
		local partyIndex = Archaeologist_PartyIndexFromName(arg1);
		Archaeologist_PartyCheckDead(partyIndex);
		--Sea.io.print(event," ",arg1);
		
	elseif (event == "PARTY_MEMBER_DISABLE") then
		local partyIndex = Archaeologist_PartyIndexFromName(arg1);
		Archaeologist_PartyCheckDead(partyIndex);
		--Sea.io.print(event," ",arg1);
	
	elseif (event == "UNIT_AURA") then
		local partyIndex = Archaeologist_PartyIndexFromUnit(arg1);
		Archaeologist_PartyCheckDead(partyIndex);
		if (arg1 == "pet") then
			Archaeologist_PetCheckDead();
		end
	end

end

-- <= == == == == == == == == == == == == =>
-- => Status Bar Initializing
-- <= == == == == == == == == == == == == =>

function Archaeologist_InitializeAddedStatusBarTexts()

	SetTextStatusBarText(PartyMemberFrame1HealthBar, PartyMemberFrame1HealthBarTextString);
	SetTextStatusBarText(PartyMemberFrame2HealthBar, PartyMemberFrame2HealthBarTextString);
	SetTextStatusBarText(PartyMemberFrame3HealthBar, PartyMemberFrame3HealthBarTextString);
	SetTextStatusBarText(PartyMemberFrame4HealthBar, PartyMemberFrame4HealthBarTextString);
	
	SetTextStatusBarText(PartyMemberFrame1ManaBar, PartyMemberFrame1ManaBarTextString);
	SetTextStatusBarText(PartyMemberFrame2ManaBar, PartyMemberFrame2ManaBarTextString);
	SetTextStatusBarText(PartyMemberFrame3ManaBar, PartyMemberFrame3ManaBarTextString);
	SetTextStatusBarText(PartyMemberFrame4ManaBar, PartyMemberFrame4ManaBarTextString);
	
	SetTextStatusBarText(TargetFrameHealthBar, TargetFrameHealthBarTextString);
	SetTextStatusBarText(TargetFrameManaBar, TargetFrameManaBarTextString);
end

-- <= == == == == == == == == == == == == =>
-- => Variable Sync
-- <= == == == == == == == == == == == == =>

function Archaeologist_VarSync_UltimateUIToVars()
	--sync internal values with ultimateui values, else use default stored, else default to 0
	for index, var in ArchaeologistVarData do
		if (var.ultimateui) and (var.min) and (var.max) then --slider
			local cosVal = getglobal(var.ultimateui);
			if (cosVal) then
				ArchaeologistVars[index] = cosVal;
			elseif (var.default) then
				ArchaeologistVars[index] = var.default;
			else
				ArchaeologistVars[index] = 0;
			end
		elseif (var.ultimateui) then
			local cosVal = getglobal(var.ultimateui.."_X");
			if (cosVal) then
				ArchaeologistVars[index] = cosVal;
			elseif (var.default) then
				ArchaeologistVars[index] = var.default;
			else
				ArchaeologistVars[index] = 0;
			end
		end
	end
end


function Archaeologist_VarSync_VarsToUltimateUI()
	--sync ultimateui values with internal values, else use default stored, else default to 0
	if(UltimateUI_RegisterConfiguration) then
		for index, var in ArchaeologistVarData do
			if (var.ultimateui) and (var.min) and (var.max) then --slider
				if(ArchaeologistVars[index]) then
					UltimateUI_UpdateValue(var.ultimateui, CSM_CHECKONOFF, 1);
					UltimateUI_SetCVar(var.ultimateui.."_X", 1);
					UltimateUI_UpdateValue(var.ultimateui, CSM_SLIDERVALUE, ArchaeologistVars[index]);
					UltimateUI_SetCVar(var.ultimateui, ArchaeologistVars[index]);
				end
			elseif (var.ultimateui) then
				if(ArchaeologistVars[index]) then
					UltimateUI_UpdateValue(var.ultimateui, CSM_CHECKONOFF, ArchaeologistVars[index]);
					UltimateUI_SetCVar(var.ultimateui.."_X", ArchaeologistVars[index]);
				end
			end
		end
		if UltimateUIMasterFrame:IsVisible() and (not UltimateUIMasterFrame_IsLoading) then
			UltimateUIMaster_DrawData();
		end
	end
end


function Archaeologist_VarSync_SavedToVars()
	--sync saved values with internal values, else use default stored, else default to 0
	for index, var in ArchaeologistVarData do
		if (ArchaeologistVars[index]) then
			--already saved
		elseif (var.default) then
			ArchaeologistVars[index] = var.default;
		else
			ArchaeologistVars[index] = 0;
		end
	end
end


function Archaeologist_VarSync_VarsToLive()
	--sync live status with internal values, else use default stored, else default to 0
	for index, var in ArchaeologistVarData do
		if (var.min) and (var.max) then --slider
			if (ArchaeologistVars[index]) and (var.func) then
				var.func(1, ArchaeologistVars[index])
			elseif (var.func) and (var.default) then
				var.func(1, var.default);
			elseif (var.func) then
				var.func(1, 0);
			end
		elseif (ArchaeologistVars[index]) and (var.func) then
			var.func(ArchaeologistVars[index])
		elseif (var.func) and (var.default) then
			var.func(var.default);
		elseif (var.func) then
			var.func(0);
		end
	end
end


-- <= == == == == == == == == == == == == =>
-- => HP Color Mod
-- <= == == == == == == == == == == == == =>

function Archaeologist_HealthBar_OnValueChanged(value, smooth)
	if (ArchaeologistVars) then
		if (ArchaeologistVars["HPCOLOR"] == 1) then
			smooth = not smooth;
		end
	end
	SavedHealthBar_OnValueChanged(value, smooth)
end

-- <= == == == == == == == == == == == == =>
-- => HPMP Text Size
-- <= == == == == == == == == == == == == =>

function Archaeologist_SetHPMPLargeTextSize(checked, size)
	ArchaeologistVars["HPMPLARGESIZE"] = size;
	local barParent = ArchaeologistStatusBars["player"].frame;
	getglobal(barParent.healthbar:GetName().."Text"):SetTextHeight(size);
	getglobal(barParent.manabar:GetName().."Text"):SetTextHeight(size);
	local scale = barParent:GetScale();
	barParent:SetScale(scale+.1);
	barParent:SetScale(scale);
	barParent = ArchaeologistStatusBars["target"].frame;
	getglobal(barParent.healthbar:GetName().."TextString"):SetTextHeight(size);
	getglobal(barParent.manabar:GetName().."TextString"):SetTextHeight(size);
	scale = barParent:GetScale();
	barParent:SetScale(scale+.1);
	barParent:SetScale(scale);
end

function Archaeologist_SetHPMPSmallTextSize(checked, size)
	ArchaeologistVars["HPMPSMALLSIZE"] = size;
	local barParent = ArchaeologistStatusBars["pet"].frame;
	getglobal(barParent.healthbar:GetName().."Text"):SetTextHeight(size);
	getglobal(barParent.manabar:GetName().."Text"):SetTextHeight(size);
	local scale = barParent:GetScale();
	barParent:SetScale(scale+.1);
	barParent:SetScale(scale);
	for i=1, 4 do
		barParent = ArchaeologistStatusBars["party"..i].frame;
		getglobal(barParent.healthbar:GetName().."TextString"):SetTextHeight(size);
		getglobal(barParent.manabar:GetName().."TextString"):SetTextHeight(size);
		scale = barParent:GetScale();
		barParent:SetScale(scale+.1);
		barParent:SetScale(scale);
	end
end

-- <= == == == == == == == == == == == == => <= == == == == == == == == == == == == =>
-- => Toggle Functions
-- <= == == == == == == == == == == == == => <= == == == == == == == == == == == == =>

-- <= == == == == == == == == == == == == =>
-- => TurnOn HP Color Function
-- <= == == == == == == == == == == == == =>

function Archaeologist_TurnOnHPColorMod(toggle)
	ArchaeologistVars["HPCOLOR"] = toggle;
end


-- <= == == == == == == == == == == == == =>
-- => TurnOn HP/MP/XP Functions
-- <= == == == == == == == == == == == == =>

function Archaeologist_TurnOnPlayerHP(toggle)
	OverrideShowStatusBarText(ArchaeologistStatusBars["player"].frame.healthbar, toggle);
	ArchaeologistVars["PLAYERHP"] = toggle;
end

function Archaeologist_RestorePlayerHP()
	Archaeologist_TurnOnPlayerHP(ArchaeologistVars["PLAYERHP"]);
end

function Archaeologist_TurnOnPlayerMP(toggle)
	OverrideShowStatusBarText(ArchaeologistStatusBars["player"].frame.manabar, toggle);
	ArchaeologistVars["PLAYERMP"] = toggle;
end

function Archaeologist_RestorePlayerMP()
	Archaeologist_TurnOnPlayerMP(ArchaeologistVars["PLAYERMP"]);
end

function Archaeologist_TurnOnPlayerXP(toggle)
	OverrideShowStatusBarText(MainMenuExpBar, toggle);
	ArchaeologistVars["PLAYERXP"] = toggle;
end

function Archaeologist_RestorePlayerXP()
	Archaeologist_TurnOnPlayerXP(ArchaeologistVars["PLAYERXP"]);
end

function Archaeologist_TurnOnPartyHP(toggle)
	for i=1, 4 do
		local bar = ArchaeologistStatusBars["party"..i].frame.healthbar;
		OverrideShowStatusBarText(bar, toggle);
	end
	ArchaeologistVars["PARTYHP"] = toggle;
end


function Archaeologist_TurnOnPartyMP(toggle) 
	for i = 1, 4 do
		local bar = ArchaeologistStatusBars["party"..i].frame.manabar
		OverrideShowStatusBarText(bar, toggle);
	end
	ArchaeologistVars["PARTYMP"] = toggle;
end


function Archaeologist_TurnOnTargetHP(toggle)
	OverrideShowStatusBarText(ArchaeologistStatusBars["target"].frame.healthbar, toggle);
	ArchaeologistVars["TARGETHP"] = toggle;
end


function Archaeologist_TurnOnTargetMP(toggle)
	OverrideShowStatusBarText(ArchaeologistStatusBars["target"].frame.manabar, toggle);
	ArchaeologistVars["TARGETMP"] = toggle;
end


function Archaeologist_TurnOnPetHP(toggle)
	OverrideShowStatusBarText(ArchaeologistStatusBars["pet"].frame.healthbar, toggle);
	ArchaeologistVars["PETHP"] = toggle;
end


function Archaeologist_TurnOnPetMP(toggle)
	OverrideShowStatusBarText(ArchaeologistStatusBars["pet"].frame.manabar, toggle);
	ArchaeologistVars["PETMP"] = toggle;
end


-- <= == == == == == == == == == == == == =>
-- => Change HP/MP/XP Values to Percent Functions
-- <= == == == == == == == == == == == == =>

function Archaeologist_TurnOnPlayerHPPercent(toggle)
	local statusBar = ArchaeologistStatusBars["player"].frame.healthbar;
	Archaeologist_TextStringPercentStatusBars[statusBar:GetName()] = toggle;
	Archaeologist_TextStatusBar_UpdateTextString(statusBar);
	ArchaeologistVars["PLAYERHPP"] = toggle;
end

function Archaeologist_TurnOnPlayerMPPercent(toggle)
	local statusBar = ArchaeologistStatusBars["player"].frame.manabar;
	Archaeologist_TextStringPercentStatusBars[statusBar:GetName()] = toggle;
	Archaeologist_TextStatusBar_UpdateTextString(statusBar);
	ArchaeologistVars["PLAYERMPP"] = toggle;
end

function Archaeologist_TurnOnPlayerXPPercent(toggle)
	Archaeologist_TextStringPercentStatusBars["MainMenuExpBar"] = toggle;
	Archaeologist_TextStatusBar_UpdateTextString(MainMenuExpBar);
	ArchaeologistVars["PLAYERXPP"] = toggle;
end

function Archaeologist_TurnOnPartyHPPercent(toggle)
	local statusBar = ArchaeologistStatusBars["party1"].frame.healthbar;
	Archaeologist_TextStringPercentStatusBars[statusBar:GetName()] = toggle;
	Archaeologist_TextStatusBar_UpdateTextString(statusBar);
	
	statusBar = ArchaeologistStatusBars["party2"].frame.healthbar;
	Archaeologist_TextStringPercentStatusBars[statusBar:GetName()] = toggle;
	Archaeologist_TextStatusBar_UpdateTextString(statusBar);
	
	statusBar = ArchaeologistStatusBars["party3"].frame.healthbar;
	Archaeologist_TextStringPercentStatusBars[statusBar:GetName()] = toggle;
	Archaeologist_TextStatusBar_UpdateTextString(statusBar);
	
	statusBar = ArchaeologistStatusBars["party4"].frame.healthbar;
	Archaeologist_TextStringPercentStatusBars[statusBar:GetName()] = toggle;
	Archaeologist_TextStatusBar_UpdateTextString(statusBar);
	
	ArchaeologistVars["PARTYHPP"] = toggle;
end

function Archaeologist_TurnOnPartyMPPercent(toggle)
	local statusBar = ArchaeologistStatusBars["party1"].frame.manabar;
	Archaeologist_TextStringPercentStatusBars[statusBar:GetName()] = toggle;
	Archaeologist_TextStatusBar_UpdateTextString(statusBar);
	
	statusBar = ArchaeologistStatusBars["party2"].frame.manabar;
	Archaeologist_TextStringPercentStatusBars[statusBar:GetName()] = toggle;
	Archaeologist_TextStatusBar_UpdateTextString(statusBar);
	
	statusBar = ArchaeologistStatusBars["party3"].frame.manabar;
	Archaeologist_TextStringPercentStatusBars[statusBar:GetName()] = toggle;
	Archaeologist_TextStatusBar_UpdateTextString(statusBar);
	
	statusBar = ArchaeologistStatusBars["party4"].frame.manabar;
	Archaeologist_TextStringPercentStatusBars[statusBar:GetName()] = toggle;
	Archaeologist_TextStatusBar_UpdateTextString(statusBar);
	
	ArchaeologistVars["PARTYMPP"] = toggle;
end

--only used to call onload since no more accurate data is provided
function Archaeologist_TurnOnTargetHPPercent(toggle)
	local statusBar = ArchaeologistStatusBars["target"].frame.healthbar;
	Archaeologist_TextStringPercentStatusBars[statusBar:GetName()] = toggle;
	Archaeologist_TextStatusBar_UpdateTextString(statusBar);
end

function Archaeologist_TurnOnTargetMPPercent(toggle)
	local statusBar = ArchaeologistStatusBars["target"].frame.manabar;
	Archaeologist_TextStringPercentStatusBars[statusBar:GetName()] = toggle;
	Archaeologist_TextStatusBar_UpdateTextString(statusBar);
	ArchaeologistVars["TARGETMPP"] = toggle;
end


function Archaeologist_TurnOnPetHPPercent(toggle)
	local statusBar = ArchaeologistStatusBars["pet"].frame.healthbar;
	Archaeologist_TextStringPercentStatusBars[statusBar:GetName()] = toggle;
	Archaeologist_TextStatusBar_UpdateTextString(statusBar);
	ArchaeologistVars["PETHPP"] = toggle;
end

function Archaeologist_TurnOnPetMPPercent(toggle)
	local statusBar = ArchaeologistStatusBars["pet"].frame.manabar;
	Archaeologist_TextStringPercentStatusBars[statusBar:GetName()] = toggle;
	Archaeologist_TextStatusBar_UpdateTextString(statusBar);
	ArchaeologistVars["PETMPP"] = toggle;
end



-- <= == == == == == == == == == == == == =>
-- => Hide HP/MP/XP Value Prefix Functions
-- <= == == == == == == == == == == == == =>

function Archaeologist_UnitFrame_UpdateManaType()
	local info = ManaBarColor[UnitPowerType(this.unit)];
	this.manabar:SetStatusBarColor(info.r, info.g, info.b);
	
	-- Update the manabar prefix only if not hidden
	if ( Archaeologist_ManaPrefixNotHidden(this) ) then
		SetTextStatusBarTextPrefix(this.manabar, info.prefix);
		TextStatusBar_UpdateTextString(this.manabar);
	end

	-- Setup newbie tooltip
	if ( this:GetName() == "PlayerFrame" ) then
		this.manabar.tooltipTitle = info.prefix;
		this.manabar.tooltipText = getglobal("NEWBIE_TOOLTIP_MANABAR"..UnitPowerType(this.unit));
	else
		this.manabar.tooltipTitle = nil;
		this.manabar.tooltipText = nil;
	end
	
end


function Archaeologist_ManaPrefixNotHidden(frame)
	if  ( (ArchaeologistVars["PLAYERMPJUSTVALUE"] == 0) and (frame == ArchaeologistStatusBars.player.frame) ) or
		( (ArchaeologistVars["PARTYMPJUSTVALUE"] == 0) and (
			(frame == ArchaeologistStatusBars.party1.frame) or
			(frame == ArchaeologistStatusBars.party2.frame) or
			(frame == ArchaeologistStatusBars.party3.frame) or
			(frame == ArchaeologistStatusBars.party4.frame)
		) ) or
		( (ArchaeologistVars["PETMPJUSTVALUE"] == 0) and (frame == ArchaeologistStatusBars.pet.frame) ) or
		( (ArchaeologistVars["TARGETMPJUSTVALUE"] == 0) and (frame == ArchaeologistStatusBars.target.frame) )
	then
		return true;
	end
end


function Archaeologist_TurnOffUnitHPPrefix(unit, toggle)
	local statusBar = ArchaeologistStatusBars[unit].frame.healthbar;
	if (toggle == 1) then
		SetTextStatusBarTextPrefix(statusBar, nil);
	else
		SetTextStatusBarTextPrefix(statusBar, TEXT(HEALTH));
	end
	Archaeologist_TextStatusBar_UpdateTextString(statusBar);
end

function Archaeologist_TurnOffUnitMPPrefix(unit, toggle)
	local statusBar = ArchaeologistStatusBars[unit].frame.manabar;
	if (toggle == 1) then
		SetTextStatusBarTextPrefix(statusBar, nil);
	else
		SetTextStatusBarTextPrefix(statusBar, ManaBarColor[UnitPowerType(unit)].prefix);
	end
	Archaeologist_TextStatusBar_UpdateTextString(statusBar);
end


function Archaeologist_TurnOffPlayerHPPrefix(toggle)
	Archaeologist_TurnOffUnitHPPrefix("player", toggle)
	ArchaeologistVars["PLAYERHPJUSTVALUE"] = toggle;
end

function Archaeologist_TurnOffPlayerMPPrefix(toggle)
	Archaeologist_TurnOffUnitMPPrefix("player", toggle)
	ArchaeologistVars["PLAYERMPJUSTVALUE"] = toggle;
end

function Archaeologist_TurnOffPlayerXPPrefix(toggle)
	local statusBar = MainMenuExpBar;
	if (toggle == 1) then
		SetTextStatusBarTextPrefix(statusBar, nil);
	else
		SetTextStatusBarTextPrefix(statusBar, TEXT(XP));
	end
	Archaeologist_TextStatusBar_UpdateTextString(statusBar);
	ArchaeologistVars["PLAYERXPJUSTVALUE"] = toggle;
end

function Archaeologist_TurnOffPartyHPPrefix(toggle)
	Archaeologist_TurnOffUnitHPPrefix("party1", toggle)
	Archaeologist_TurnOffUnitHPPrefix("party2", toggle)
	Archaeologist_TurnOffUnitHPPrefix("party3", toggle)
	Archaeologist_TurnOffUnitHPPrefix("party4", toggle)
	ArchaeologistVars["PARTYHPJUSTVALUE"] = toggle;
end

function Archaeologist_TurnOffPartyMPPrefix(toggle)
	Archaeologist_TurnOffUnitMPPrefix("party1", toggle)
	Archaeologist_TurnOffUnitMPPrefix("party2", toggle)
	Archaeologist_TurnOffUnitMPPrefix("party3", toggle)
	Archaeologist_TurnOffUnitMPPrefix("party4", toggle)
	ArchaeologistVars["PARTYMPJUSTVALUE"] = toggle;
end

function Archaeologist_TurnOffTargetHPPrefix(toggle)
	Archaeologist_TurnOffUnitHPPrefix("target", toggle)
	ArchaeologistVars["TARGETHPJUSTVALUE"] = toggle;
end

function Archaeologist_TurnOffTargetMPPrefix(toggle)
	Archaeologist_TurnOffUnitMPPrefix("target", toggle)
	ArchaeologistVars["TARGETMPJUSTVALUE"] = toggle;
end


function Archaeologist_TurnOffPetHPPrefix(toggle)
	Archaeologist_TurnOffUnitHPPrefix("pet", toggle)
	ArchaeologistVars["PETHPJUSTVALUE"] = toggle;
end

function Archaeologist_TurnOffPetMPPrefix(toggle)
	Archaeologist_TurnOffUnitMPPrefix("pet", toggle)
	ArchaeologistVars["PETMPJUSTVALUE"] = toggle;
end


-- <= == == == == == == == == == == == == =>
-- => Status HP/MP/XP Bar Overrides
-- <= == == == == == == == == == == == == =>

function OverrideShowStatusBarText(bar, toggle)
	if (toggle == 1) then
		SetStatusBarTextOverride(bar, 1);
	else
		SetStatusBarTextOverride(bar, nil);
	end
end

--[[unused.. yet
function OverrideHideStatusBarText(bar, toggle)
	if (toggle == 1) then
		SetStatusBarTextOverride(bar, 0);
	else
		SetStatusBarTextOverride(bar, nil);
	end
end
]]--

--sets the override for StatusBarTexts.
--override = nil removes override, 0 sets to hide, 1 sets to show
function SetStatusBarTextOverride(bar, override)
	if(not bar) then
		return;
	end
	if(override == "1" or override == 1 or override == "show") then
		bar.override = "show";
		--UIOptionsFrameCheckButtons["STATUS_BAR_TEXT"].value = "1"
		ShowTextStatusBarText(bar);
	elseif(override == "0" or override == 0 or override == "hide") then
		bar.override = "hide";
		HideTextStatusBarText(bar);
	else
		bar.override = nil;
		HideTextStatusBarText(bar);
	end
end


--updates old lockShow to the new override notation
local function ConvertLockShowToOverrideSyntax(bar)
	if (bar.textLockable) then
		bar.textLockable = nil;
	end
	if (bar.lockShow) then
		if (bar.lockShow == 1) then
			bar.override = "show";
		end
		bar.lockShow = nil;
	end
end


--allows the setting of textStatusBar.oneText to override other values if value = 1
--used to hide hp text of a ghost
--also added optional percents
function Archaeologist_TextStatusBar_UpdateTextString(textStatusBar)
	if ( not textStatusBar ) then
		textStatusBar = this;
	end
	local string = textStatusBar.TextString;
	if(string) then
		local value = textStatusBar:GetValue();
		local valueMin, valueMax = textStatusBar:GetMinMaxValues();
		if ( valueMax > 0 ) then
			textStatusBar:Show();
			if ( value == 0 and textStatusBar.zeroText ) then
				string:SetText(textStatusBar.zeroText);
				textStatusBar.isZero = 1;
				ShowTextStatusBarText(textStatusBar);
			elseif ( value == 1 and textStatusBar.oneText ) then
				string:SetText(textStatusBar.oneText);
				textStatusBar.isOne = 1;
				string:Show();
				ShowTextStatusBarText(textStatusBar);
			else
				textStatusBar.isZero = nil;
				textStatusBar.isOne = nil;
				
				local stringText = "";
				if (textStatusBar == TargetFrame.healthbar) and (valueMax == 100) then
					stringText = Sea.math.round(value / valueMax * 100).."%";
				elseif (Archaeologist_TextStringPercentStatusBars[textStatusBar:GetName()] == 1) then
					stringText = Sea.math.round(value / valueMax * 100).."%";
				else
					stringText = value.." / "..valueMax;
				end
				if ( textStatusBar.prefix ) then
					stringText = textStatusBar.prefix.." "..stringText;
				end
				string:SetText(stringText);
				
				if (textStatusBar.override == "show") or (textStatusBar:GetLeft()) and (MouseIsOver(textStatusBar)) then
					ShowTextStatusBarText(textStatusBar);
				else
					HideTextStatusBarText(textStatusBar);
				end
			end
		else
			textStatusBar:Hide();
		end
	end
end


--removes lockShow and adds override
function Archaeologist_ShowTextStatusBarText(bar)
	if ( bar and bar.TextString ) then
		ConvertLockShowToOverrideSyntax(bar);
		if (bar.override ~= "hide") then
			bar.TextString:Show();
		end
	end
end

--removes old lockShow, adds override, adds visibility for isOne, and removes UIOptions check
--effectively breaks the 'Show HP/Mana/XP Always Vislible' in the default UIOptions
function Archaeologist_HideTextStatusBarText(bar)
	if ( bar and bar.TextString ) then
		ConvertLockShowToOverrideSyntax(bar);
		if (bar.override == "hide") then
			bar.TextString:Hide();
		elseif (bar.isZero) or (bar.isOne) or (bar.override == "show") then -- or (MouseIsOver(bar)) then
			bar.TextString:Show();
		else
			bar.TextString:Hide();
		end
	end
end


--sets bar.oneText
function SetTextStatusBarTextOneText(bar, oneText)
	if ( bar and bar.TextString ) then
		bar.oneText = oneText;
	end
end

function Archaeologist_CharacterFrame_OnShow()
	PlaySound("igCharacterInfoOpen");
	SetPortraitTexture(CharacterFramePortrait, "player");
	CharacterNameText:SetText(UnitPVPName("player"));
	UpdateMicroButtons();
	--ShowTextStatusBarText(PlayerFrameHealthBar);
	--ShowTextStatusBarText(PlayerFrameManaBar);
	--ShowTextStatusBarText(MainMenuExpBar);
	OverrideShowStatusBarText(PlayerFrameHealthBar, 1);
	OverrideShowStatusBarText(PlayerFrameManaBar, 1);
	OverrideShowStatusBarText(MainMenuExpBar, 1);
end

function Archaeologist_CharacterFrame_OnHide()
	PlaySound("igCharacterInfoClose");
	UpdateMicroButtons();
	--HideTextStatusBarText(PlayerFrameHealthBar);
	--HideTextStatusBarText(PlayerFrameManaBar);
	--HideTextStatusBarText(MainMenuExpBar);
	Archaeologist_RestorePlayerHP();
	Archaeologist_RestorePlayerMP();
	Archaeologist_RestorePlayerXP();
end

-- <= == == == == == == == == == == == == =>
-- => Dead/Offline/Ghost Status Overrides
-- <= == == == == == == == == == == == == =>


function Archaeologist_TargetCheckDead()
	local unit = "target";
	local healthbar = ArchaeologistStatusBars[unit].frame.healthbar;
	local manabar = ArchaeologistStatusBars[unit].frame.manabar;
	local statusText = ArchaeologistStatusBars[unit].statusText;
	Archaeologist_UnitCheckDead(unit, statusText, healthbar, manabar);
end


function Archaeologist_PartyCheckDead(partyIndex)
	if (type(partyIndex) ~= "number")  then
		return;
	end
	local unit = "party"..partyIndex;
	local healthbar = ArchaeologistStatusBars[unit].frame.healthbar;
	local manabar = ArchaeologistStatusBars[unit].frame.manabar;
	local statusText = ArchaeologistStatusBars[unit].statusText;
	Archaeologist_UnitCheckDead(unit, statusText, healthbar, manabar);
end


function Archaeologist_UpdatePartyMembersDead()
	if (GetNumPartyMembers() > 0) then
		for i=1, GetNumPartyMembers() do
			Archaeologist_PartyCheckDead(i);
		end
	end
end


function Archaeologist_PlayerCheckDead()
	local unit = "player";
	local healthbar = ArchaeologistStatusBars[unit].frame.healthbar;
	local manabar = ArchaeologistStatusBars[unit].frame.manabar;
	local statusText = ArchaeologistStatusBars[unit].statusText;
	Archaeologist_UnitCheckDead(unit, statusText, healthbar, manabar);
end


function Archaeologist_PetCheckDead()
	local unit = "pet";
	local healthbar = ArchaeologistStatusBars[unit].frame.healthbar;
	local manabar = ArchaeologistStatusBars[unit].frame.manabar;
	local statusText = ArchaeologistStatusBars[unit].statusText;
	Archaeologist_UnitCheckDead(unit, statusText, healthbar, manabar);
end


function Archaeologist_UnitCheckDead(unit, statusText, healthbar, manabar)
	--adds Dead text if unit is Dead
	--adds Offline text if unit is a player and not connected
	--adds Ghost/Wisp text if unit is a player
	--Sea.io.print("CheckDead: "..unit);
	--Sea.io.print(this:GetName());
	
	if ( UnitIsDead(unit) ) and ( Archaeologist_UnitIsConnected(unit) ) then
	
		statusText:SetText(DEAD);
		statusText:Show();
		
		--hide health/mana if dead 
		SetTextStatusBarTextZeroText(healthbar, "");
		SetTextStatusBarTextZeroText(manabar, "");
		
	elseif ( UnitIsPlayer(unit) ) and ( not Archaeologist_UnitIsConnected(unit) ) then
		
		statusText:SetText(PLAYER_OFFLINE);
		healthbar:Hide();
		manabar:Hide();
		--!!!!!!!!!!!
		--^^ add a status bar hook to change this.
		--!!!!!!!!!!!
		statusText:Show();
		
		--hide health/mana if offline 
		SetTextStatusBarTextZeroText(healthbar, "");
		SetTextStatusBarTextZeroText(manabar, "");
		
	elseif ( UnitIsGhost(unit) ) then
	
		if ( UnitRace(unit) == "Night Elf" ) then
			statusText:SetText(PLAYER_WISP);
		else
			statusText:SetText(PLAYER_GHOST);
		end
		statusText:Show();

		--hide health/mana if ghost 
		SetTextStatusBarTextOneText(healthbar, "");
		SetTextStatusBarTextZeroText(manabar, "");
		
	else
	
		statusText:Hide();
		
		--show health/mana if not dead, offline or ghost
		SetTextStatusBarTextZeroText(healthbar, nil);
		SetTextStatusBarTextOneText(healthbar, nil);
		SetTextStatusBarTextZeroText(manabar, nil);
		
		--reset to override show
		ShowTextStatusBarText(healthbar);
		
	end
	
	TextStatusBar_UpdateTextString(healthbar);
	TextStatusBar_UpdateTextString(manabar);
	
end

function Archaeologist_UnitIsConnected(unit)
	ArchaeologistTooltip:SetUnit(unit);
	local tooltipOfflineLine = Sea.wow.tooltip.scan("ArchaeologistTooltip")[3];
	Sea.wow.tooltip.clear("ArchaeologistTooltip");
	local tooltipOfflineText;
	if ( tooltipOfflineLine ) then
		tooltipOfflineText = tooltipOfflineLine.left;
	end
	if (tooltipOfflineText == PLAYER_OFFLINE) then
		return;
	end
	return UnitIsConnected(unit);
end


-- <= == == == == == == == == == == == == =>
-- => Party and Pet Buffs
-- <= == == == == == == == == == == == == =>

function Archaeologist_PartyMemberFrame_RefreshBuffs()
	
	local texture;
	if ( ArchaeologistVars["PBUFFS"] == 1 ) then
		for i=1, ArchaeologistVarData["PBUFFNUM"].max do
			getglobal(this:GetName().."NewBuff"..i):Hide();
		end
	else
		for i=1, ArchaeologistVarData["PBUFFNUM"].max do
			texture = UnitBuff("party"..this:GetID(), i);
			if ( texture ) and (i <= ArchaeologistVars["PBUFFNUM"]) then
				getglobal(this:GetName().."NewBuff"..i.."Icon"):SetTexture(texture);
				getglobal(this:GetName().."NewBuff"..i):SetID(i);
				getglobal(this:GetName().."NewBuff"..i):Show();
			else
				getglobal(this:GetName().."NewBuff"..i):Hide();
			end
		end
	end
	
	if ( ArchaeologistVars["PDEBUFFS"] == 1 ) then
		for i=1, ArchaeologistVarData["PDEBUFFNUM"].max do
			getglobal(this:GetName().."NewDebuff"..i):Hide();
		end
	else
		local texture;
		for i=1, ArchaeologistVarData["PDEBUFFNUM"].max do
			texture = UnitDebuff("party"..this:GetID(), i);
			if (texture) and (i <= ArchaeologistVars["PDEBUFFNUM"]) then
				getglobal(this:GetName().."NewDebuff"..i.."Icon"):SetTexture(texture);
				getglobal(this:GetName().."NewDebuff"..i):SetID(i);
				getglobal(this:GetName().."NewDebuff"..i):Show();
			else
				getglobal(this:GetName().."NewDebuff"..i):Hide();
			end
		end
	end
	
end


function Archaeologist_PetFrame_RefreshBuffs()
	
	local texture;
	if ( ArchaeologistVars["PTBUFFS"] == 1 ) then
		for i=1, ArchaeologistVarData["PTBUFFNUM"].max do
			getglobal(this:GetName().."NewBuff"..i):Hide();
		end
	else
		for i=1, ArchaeologistVarData["PTBUFFNUM"].max do
			texture = UnitBuff("pet", i);
			if ( texture ) and (i <= ArchaeologistVars["PTBUFFNUM"]) then
				getglobal(this:GetName().."NewBuff"..i.."Icon"):SetTexture(texture);
				getglobal(this:GetName().."NewBuff"..i):SetID(i);
				getglobal(this:GetName().."NewBuff"..i):Show();
			else
				getglobal(this:GetName().."NewBuff"..i):Hide();
			end
		end
	end
	
	if ( ArchaeologistVars["PTDEBUFFS"] == 1 ) then
		for i=1, ArchaeologistVarData["PTDEBUFFNUM"].max do
			getglobal(this:GetName().."NewDebuff"..i):Hide();
		end
	else
		local texture;
		for i=1, ArchaeologistVarData["PTDEBUFFNUM"].max do
			texture = UnitDebuff("pet", i);
			if (texture) and (i <= ArchaeologistVars["PTDEBUFFNUM"]) then
				getglobal(this:GetName().."NewDebuff"..i.."Icon"):SetTexture(texture);
				getglobal(this:GetName().."NewDebuff"..i):SetID(i);
				getglobal(this:GetName().."NewDebuff"..i):Show();
			else
				getglobal(this:GetName().."NewDebuff"..i):Hide();
			end
		end
	end
	
end


function Archaeologist_UpdatePartyMemberBuffs()
	local tempThis = this;
	for i=1, MAX_PARTY_MEMBERS do
        if ( GetPartyMember(i) ) then
			this = getglobal("PartyMemberFrame"..i);
            Archaeologist_PartyMemberFrame_RefreshBuffs()
        end
    end
	this = tempThis;
end

function Archaeologist_UpdatePetBuffs()
	this = PetFrame;
	Archaeologist_PetFrame_RefreshBuffs()
end


function Archaeologist_PartyMemberBuffs_Update()
	--only show buff tooltip on mouseover if buffs are hidden
	if (arg1 == "pet") then
		return (ArchaeologistVars["PTBUFFS"] == 1);
	end
	return (ArchaeologistVars["PBUFFS"] == 1);
end


function Archaeologist_TurnOffPartyBuffs(toggle)
	ArchaeologistVars["PBUFFS"] = toggle;
	Archaeologist_UpdatePartyMemberBuffs();
end

function Archaeologist_TurnOffPetBuffs(toggle)
	ArchaeologistVars["PTBUFFS"] = toggle;
	Archaeologist_UpdatePetBuffs();
end


function Archaeologist_TurnOffPartyDebuffs(toggle)
	ArchaeologistVars["PDEBUFFS"] = toggle;
	Archaeologist_UpdatePartyMemberBuffs();
end

function Archaeologist_TurnOffPetDebuffs(toggle)
	ArchaeologistVars["PTDEBUFFS"] = toggle;
	Archaeologist_UpdatePetBuffs();
end


function Archaeologist_SetPartyBuffs(toggle, count)
	if (count) then
		ArchaeologistVars["PBUFFNUM"] = count;
		Archaeologist_UpdatePartyMemberBuffs();
	end
end

function Archaeologist_SetPetBuffs(toggle, count)
	if (count) then
		ArchaeologistVars["PTBUFFNUM"] = count;
		Archaeologist_UpdatePetBuffs();
	end
end


function Archaeologist_SetPartyDebuffs(toggle, count)
	if (count) then
		ArchaeologistVars["PDEBUFFNUM"] = count;
		Archaeologist_UpdatePartyMemberBuffs();
	end
end

function Archaeologist_SetPetDebuffs(toggle, count)
	if (count) then
		ArchaeologistVars["PTDEBUFFNUM"] = count;
		Archaeologist_UpdatePetBuffs();
	end
end

-- <= == == == == == == == == == == == == =>
-- => Target Buffs
-- <= == == == == == == == == == == == == =>

function Archaeologist_HideOrigTargetBuffs()
	for i=1, MAX_TARGET_DEBUFFS do
		getglobal("TargetFrameBuff"..i):Hide();
		getglobal("TargetFrameDebuff"..i):Hide();
	end
end

function Archaeologist_TargetDebuffButton_Update()
	-- Position buffs depending on whether the targeted unit is friendly or not
	Archaeologist_TargetBuffs_SetVerticalAlignment( UnitIsFriend("player", "target") );
	
	local debuff, debuffButton, buff, buffButton;
	local button;
	for i=1, ArchaeologistVarData["TBUFFNUM"].max do
		buff = UnitBuff("target", i);
		button = getglobal("TargetFrameNewBuff"..i);
		if ( buff ) and (i <= ArchaeologistVars["TBUFFNUM"]) and (ArchaeologistVars["TBUFFS"] == 0) then
			getglobal("TargetFrameNewBuff"..i.."Icon"):SetTexture(buff);
			button:Show();
			button.id = i;
		else
			button:Hide();
		end
	end
	for i=1, ArchaeologistVarData["TDEBUFFNUM"].max do
		debuff = UnitDebuff("target", i);
		button = getglobal("TargetFrameNewDebuff"..i);
		if ( debuff ) and (i <= ArchaeologistVars["TDEBUFFNUM"]) and (ArchaeologistVars["TDEBUFFS"] == 0) then
			getglobal("TargetFrameNewDebuff"..i.."Icon"):SetTexture(debuff);
			button:Show();
		else
			button:Hide();
		end
		button.id = i;
	end
end

function Archaeologist_TargetBuffs_SetVerticalAlignment(buffsOnTop)
	local leftPos;
	if (ArchaeologistVars["TBUFFALT"] == 1) then
		leftPos = 100;
	else
		leftPos = 5;
	end
	TargetFrameNewBuff1:ClearAllPoints();
	TargetFrameNewDebuff1:ClearAllPoints();
	if (buffsOnTop) then
		-- Move Buffs to Top
		TargetFrameNewBuff1:SetPoint("TOPLEFT", "TargetFrame", "BOTTOMLEFT", leftPos, 32);
		TargetFrameNewDebuff1:SetPoint("TOPRIGHT", "TargetFrameNewBuff1", "BOTTOMRIGHT", 0, -2);
		Archaeologist_TargetBuffsAreOnTop = true;
	else
		-- Move Buffs to Bottom
		TargetFrameNewDebuff1:SetPoint("TOPLEFT", "TargetFrame", "BOTTOMLEFT", leftPos, 32);
		TargetFrameNewBuff1:SetPoint("TOPRIGHT", "TargetFrameNewDebuff1", "BOTTOMRIGHT", 0, -2);
		Archaeologist_TargetBuffsAreOnTop = false;
	end
end

function Archaeologist_TargetBuffs_SetAltHorizontalAlignment(toggle)
	local button, point, relativePoint, leftOffset;
	if (toggle == 1) then
		point = "RIGHT";
		relativePoint = "LEFT";
		leftOffset = -3;
	else
		point = "LEFT";
		relativePoint = "RIGHT";
		leftOffset = 3;
	end
	
	ArchaeologistVars["TBUFFALT"] = toggle;
	
	Archaeologist_TargetBuffs_SetVerticalAlignment(Archaeologist_TargetBuffsAreOnTop);
	
	for i=2, ArchaeologistVarData["TBUFFNUM"].max do
		button = getglobal("TargetFrameNewBuff"..i);
		button:ClearAllPoints();
		button:SetPoint(point, "TargetFrameNewBuff"..(i-1), relativePoint, leftOffset, 0);
	end
	for i=2, ArchaeologistVarData["TDEBUFFNUM"].max do
		button = getglobal("TargetFrameNewDebuff"..i);
		button:ClearAllPoints();
		button:SetPoint(point, "TargetFrameNewDebuff"..(i-1), relativePoint, leftOffset, 0);
	end
end

function Archaeologist_TurnOffTargetBuffs(toggle)
	ArchaeologistVars["TBUFFS"] = toggle;
	TargetDebuffButton_Update();
end

function Archaeologist_TurnOffTargetDebuffs(toggle)
	ArchaeologistVars["TDEBUFFS"] = toggle;
	TargetDebuffButton_Update();
end


function Archaeologist_SetTargetBuffs(toggle, count)
	if (count) then
		ArchaeologistVars["TBUFFNUM"] = count;
		TargetDebuffButton_Update();
	end
end


function Archaeologist_SetTargetDebuffs(toggle, count)
	if (count) then
		ArchaeologistVars["TDEBUFFNUM"] = count;
		TargetDebuffButton_Update();
	end
end

-- <= == == == == == == == == == == == == =>
-- => Alternate Options
-- <= == == == == == == == == == == == == =>

function Archaeologist_SetAltDebuffLocation(toggle)
	ArchaeologistVars["DEBUFFALT"] = toggle;
	if (toggle == 1) then
		PetFrameNewDebuff1:ClearAllPoints();
		PetFrameNewDebuff1:SetPoint("TOP", "PetFrameNewBuff1", "BOTTOM", 0, -2);
		for i=1, 4 do
			getglobal("PartyMemberFrame"..i.."NewDebuff1"):ClearAllPoints();
			getglobal("PartyMemberFrame"..i.."NewDebuff1"):SetPoint("TOP", "PartyMemberFrame"..i.."NewBuff1", "BOTTOM", 0, -2);
		end
	else
		PetFrameNewDebuff1:ClearAllPoints();
		PetFrameNewDebuff1:SetPoint("TOPLEFT", "PetFrame", "TOPLEFT", 120, -24);
		for i=1, 4 do
			getglobal("PartyMemberFrame"..i.."NewDebuff1"):ClearAllPoints();
			getglobal("PartyMemberFrame"..i.."NewDebuff1"):SetPoint("TOPLEFT", "PartyMemberFrame"..i, "TOPLEFT", 124, -14);
		end
	end
	
	Archaeologist_SetPetFrameHappinessLocation();
end


function Archaeologist_SetAltHPMPLocation(toggle)
	ArchaeologistVars["HPMPALT"] = toggle;
	local bar;
	local bartext;
	local bartextstring;
	local stringSuffix;
	local point;
	local relativePoint;
	local justify;
	local width = 20;
	local x;
	local y;

	for unit, data in ArchaeologistStatusBars do
		if (toggle == 1) then
			if (unit == "target") then
				point = "RIGHT";
				relativePoint = "LEFT";
				justify = "RIGHT";
			else
				point = "LEFT";
				relativePoint = "RIGHT";
				justify = "LEFT";
				
			end
		else
			point = "CENTER";
			relativePoint = "CENTER";
			justify = "CENTER";
		end
		if (unit == "pet") or (unit == "player") then
			stringSuffix = "";
		else
			stringSuffix = "String";
		end
		x = 0;
		y = 2;
		
		bar = getglobal(data.frame:GetName().."HealthBar");
		bartext = getglobal(data.frame:GetName().."HealthBarText");
		bartextstring = getglobal(data.frame:GetName().."HealthBarText"..stringSuffix);
		
		bartext:SetHeight(bar:GetHeight());
		bartext:SetWidth(bar:GetWidth()+width);
		bartext:ClearAllPoints();
		bartext:SetPoint(point, bar:GetName(), relativePoint, x, y);
		bartextstring:SetJustifyH(justify);
		
		y = -2;
		
		bar = getglobal(data.frame:GetName().."ManaBar");
		bartext = getglobal(data.frame:GetName().."ManaBarText");
		bartextstring = getglobal(data.frame:GetName().."ManaBarText"..stringSuffix);
		
		bartext:SetHeight(bar:GetHeight());
		bartext:SetWidth(bar:GetWidth()+width);
		bartext:ClearAllPoints();
		bartext:SetPoint(point, bar:GetName(), relativePoint, x, y);
		bartextstring:SetJustifyH(justify);
	end
	
	Archaeologist_SetPetFrameHappinessLocation();

end


function Archaeologist_SetPetFrameHappinessLocation()
	if (ArchaeologistVars["HPMPALT"] == 1) or (ArchaeologistVars["DEBUFFALT"] == 0) then
		--alt position
		PetFrameHappiness:ClearAllPoints();
		PetFrameHappiness:SetPoint("TOPRIGHT", "PetFrame", "BOTTOMLEFT", 8, 15);
	else
		--normal position
		PetFrameHappiness:ClearAllPoints();
		PetFrameHappiness:SetPoint("LEFT", "PetFrame", "RIGHT", -7, -4);
	end

end



-- <= == == == == == == == == == == == == =>
-- => Helpful Funcs
-- <= == == == == == == == == == == == == =>

-- Doesn't blow up if used before the frame position can be obtained (returns nil)
function Archaeologist_MouseIsOver(frame, ...)
	if (frame:GetLeft()) then
		return true;
	end
end

-- 1 => 0, 0 => 1
function BinaryInvert(oneZero)
	if oneZero == 1 then
		return 0;
	else 
		return 1;
	end
end

function Archaeologist_PartyIndexFromName(name)
	for i=1, GetNumPartyMembers() do
		if ( name == UnitName("party"..i) ) then
			return i;
		end
	end
end


function Archaeologist_PartyIndexFromUnit(unit)
	if (type(unit) == "string") then
		if ( strsub(unit,0, string.len(unit)-1) == "party" ) then
			local partyIndex = tonumber( strsub(unit,string.len(unit)) );
			return partyIndex;
		end
	end
end


function Archaeologist_PartyIndexFromFrame(frame)
	local frameName = frame:GetName();
	if (frameName) then
		if ( strsub(frameName,0, string.len(frameName)-1) == "PartyMemberFrame" ) then
			local frameIndex = frame:GetID();
			if (frameIndex > 0) then
				return frameIndex;
			end
		end
	end
end


-- <= == == == == == == == == == == == == =>
-- => Options Via UltimateUI Or Slash Commands
-- <= == == == == == == == == == == == == =>

function Archaeologist_RegisterForUltimateUI()
		
	UltimateUI_RegisterConfiguration(
		"UUI_ARCHAEOLOGIST",
		"SECTION",
		TEXT(ARCHAEOLOGIST_CONFIG_SEP),
		TEXT(ARCHAEOLOGIST_CONFIG_SEP_INFO)
	);
	
	-- <= == == == == == == == == == == == =>
	-- => Looped Registering
	-- <= == == == == == == == == == == == =>
	
	local varPrefixes = { "PLAYER", "PARTY", "PET", "TARGET" };
	
	table.sort(varPrefixes);
	for index, varPrefix in varPrefixes do
	
		UltimateUI_RegisterConfiguration(
			"UUI_ARCHAEOLOGIST_"..varPrefix.."_SEPARATOR",
			"SEPARATOR",
			TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix.."_SEP")),
			TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix.."_SEP_INFO"))
		);
		
		local keyList = Sea.table.getKeyList(ArchaeologistVarData);
		local sorter = function(a,b) 
			if ( type(getglobal("ARCHAEOLOGIST_CONFIG_"..a)) == type(getglobal("ARCHAEOLOGIST_CONFIG_"..b)) and type(getglobal("ARCHAEOLOGIST_CONFIG_"..a)) ~= "nil") then 
				return (getglobal("ARCHAEOLOGIST_CONFIG_"..a) < getglobal("ARCHAEOLOGIST_CONFIG_"..b));
			else
				return false;
			end
		end
		table.sort(keyList, sorter);

		for k, index in keyList do
			local var = ArchaeologistVarData[index];
			if (type(index) == "string") then
				if (strsub(index, 0, string.len(varPrefix)) == varPrefix) then

					UltimateUI_RegisterConfiguration(
						ArchaeologistVarData[index].ultimateui,					--CVar
						"CHECKBOX",											--Things to use
						TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..index)),
						TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..index.."_INFO")),
						ArchaeologistVarData[index].func,					--Callback
						ArchaeologistVars[index]							--Default Checked/Unchecked
					);
					
				end
			end
		end
		
	end
	
	-- <= == == == == == == == == == == == =>
	-- => Alternate Options Registering
	-- <= == == == == == == == == == == == =>

	local varPrefix = "ALTOPTS";
	
	UltimateUI_RegisterConfiguration(
		"UUI_ARCHAEOLOGIST_"..varPrefix.."_SEPARATOR",
		"SEPARATOR",
		TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix.."_SEP")),
		TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix.."_SEP_INFO"))
	);
	
	varPrefixes = { "HPCOLOR", "DEBUFFALT", "TBUFFALT", "HPMPALT" };
	
	for index, varPrefix in varPrefixes do
	
		UltimateUI_RegisterConfiguration(
			ArchaeologistVarData[varPrefix].ultimateui,						--CVar
			"CHECKBOX",													--Things to use
			TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix)),
			TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix.."_INFO")),
			ArchaeologistVarData[varPrefix].func,						--Callback
			ArchaeologistVars[varPrefix]								--Default Checked/Unchecked
		);
	
	end
	
	varPrefixes = { "HPMPLARGESIZE", "HPMPSMALLSIZE" };
	
	for index, varPrefix in varPrefixes do
	
		UltimateUI_RegisterConfiguration(
			ArchaeologistVarData[varPrefix].ultimateui,		-- CVar
			"SLIDER",					-- Type
			TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix)),		-- Short description
			TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix.."_INFO")),	-- Long description
			ArchaeologistVarData[varPrefix].func,	-- Callback
			1,						-- Default Checked/Unchecked
			ArchaeologistVarData[varPrefix].default,		-- Default slider value
			ArchaeologistVarData[varPrefix].min,		-- Minimum slider value
			ArchaeologistVarData[varPrefix].max, 		-- max value
			TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix.."_SLIDERTEXT")), 	-- slider "header" text
			1,						-- Slider steps
			1,						-- Text on slider
			"",						-- slider text append
			1						-- slider multiplier
		);
			
	end
	
	-- <= == == == == == == == == == == == =>
	-- => Party Buff Registering
	-- <= == == == == == == == == == == == =>
	
	varPrefix = "PARTYBUFFS";
	
	UltimateUI_RegisterConfiguration(
		"UUI_ARCHAEOLOGIST_"..varPrefix.."_SEPARATOR",
		"SEPARATOR",
		TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix.."_SEP")),
		TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix.."_SEP_INFO"))
	);
	
	varPrefixes = { "PBUFF", "PDEBUFF" };
	
	for index, varPrefix in varPrefixes do
	
		UltimateUI_RegisterConfiguration(
			ArchaeologistVarData[varPrefix.."S"].ultimateui,					--CVar
			"CHECKBOX",														--Things to use
			TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix.."S")),
			TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix.."S".."_INFO")),
			ArchaeologistVarData[varPrefix.."S"].func,						--Callback
			ArchaeologistVars[varPrefix.."S"]								--Default Checked/Unchecked
		);
	
		UltimateUI_RegisterConfiguration(
			ArchaeologistVarData[varPrefix.."NUM"].ultimateui,								-- CVar
			"SLIDER",														-- Type
			TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix.."_NUM")),				-- Short description
			TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix.."_NUM_INFO")),		-- Long description
			ArchaeologistVarData[varPrefix.."NUM"].func,								-- Callback
			1,																-- Default Checked/Unchecked
			ArchaeologistVars[varPrefix.."NUM"],										-- Default slider value
			ArchaeologistVarData[varPrefix.."NUM"].min,									-- Minimum slider value
			ArchaeologistVarData[varPrefix.."NUM"].max,									-- max value
			TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix.."_NUM_SLIDER_TEXT")),	-- slider "header" text
			1,						-- Slider steps
			1,						-- Text on slider
			"",						-- slider text append
			1						-- slider multiplier
		);
		
	end
	
	-- <= == == == == == == == == == == == =>
	-- => Pet Buff Registering
	-- <= == == == == == == == == == == == =>
	
	varPrefix = "PETBUFFS";
	
	UltimateUI_RegisterConfiguration(
		"UUI_ARCHAEOLOGIST_"..varPrefix.."_SEPARATOR",
		"SEPARATOR",
		TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix.."_SEP")),
		TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix.."_SEP_INFO"))
	);
	
	varPrefixes = { "PTBUFF", "PTDEBUFF" };
	
	for index, varPrefix in varPrefixes do
	
		UltimateUI_RegisterConfiguration(
			ArchaeologistVarData[varPrefix.."S"].ultimateui,					--CVar
			"CHECKBOX",														--Things to use
			TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix.."S")),
			TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix.."S".."_INFO")),
			ArchaeologistVarData[varPrefix.."S"].func,						--Callback
			ArchaeologistVars[varPrefix.."S"]								--Default Checked/Unchecked
		);
	
		UltimateUI_RegisterConfiguration(
			ArchaeologistVarData[varPrefix.."NUM"].ultimateui,								-- CVar
			"SLIDER",														-- Type
			TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix.."NUM")),				-- Short description
			TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix.."NUM_INFO")),		-- Long description
			ArchaeologistVarData[varPrefix.."NUM"].func,								-- Callback
			1,																-- Default Checked/Unchecked
			ArchaeologistVars[varPrefix.."NUM"],										-- Default slider value
			ArchaeologistVarData[varPrefix.."NUM"].min,									-- Minimum slider value
			ArchaeologistVarData[varPrefix.."NUM"].max,									-- max value
			TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix.."NUM_SLIDER_TEXT")),	-- slider "header" text
			1,						-- Slider steps
			1,						-- Text on slider?
			"",						-- slider text append
			1						-- slider multiplier
		);
		
	end
	
	-- <= == == == == == == == == == == == =>
	-- => Target Buff Registering
	-- <= == == == == == == == == == == == =>
	
	varPrefix = "TARGETBUFFS";
	
	UltimateUI_RegisterConfiguration(
		"UUI_ARCHAEOLOGIST_"..varPrefix.."_SEPARATOR",
		"SEPARATOR",
		TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix.."_SEP")),
		TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix.."_SEP_INFO"))
	);
	
	varPrefixes = { "TBUFF", "TDEBUFF" };
	
	for index, varPrefix in varPrefixes do
	
		UltimateUI_RegisterConfiguration(
			ArchaeologistVarData[varPrefix.."S"].ultimateui,					--CVar
			"CHECKBOX",														--Things to use
			TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix.."S")),
			TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix.."S".."_INFO")),
			ArchaeologistVarData[varPrefix.."S"].func,						--Callback
			ArchaeologistVars[varPrefix.."S"]								--Default Checked/Unchecked
		);
	
		UltimateUI_RegisterConfiguration(
			ArchaeologistVarData[varPrefix.."NUM"].ultimateui,								-- CVar
			"SLIDER",														-- Type
			TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix.."NUM")),				-- Short description
			TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix.."NUM_INFO")),		-- Long description
			ArchaeologistVarData[varPrefix.."NUM"].func,								-- Callback
			1,																-- Default Checked/Unchecked
			ArchaeologistVars[varPrefix.."NUM"],										-- Default slider value
			ArchaeologistVarData[varPrefix.."NUM"].min,									-- Minimum slider value
			ArchaeologistVarData[varPrefix.."NUM"].max,									-- max value
			TEXT(getglobal("ARCHAEOLOGIST_CONFIG_"..varPrefix.."NUM_SLIDER_TEXT")),	-- slider "header" text
			1,						-- Slider steps
			1,						-- Text on slider?
			"",						-- slider text append
			1						-- slider multiplier
		);
		
	end
	
end


function Archaeologist_RegisterForSlashCommands()
	
	SlashCmdList["ARCHHELP"] = Archaeologist_Slash_ArchHelp;
    SLASH_ARCHHELP1 = "/archhelp";
	
	SlashCmdList["ARCHPLAYER"] = Archaeologist_Slash_ArchPlayer;
    SLASH_ARCHPLAYER1 = "/archplayer";
	
	SlashCmdList["ARCHPARTY"] = Archaeologist_Slash_ArchParty;
    SLASH_ARCHPARTY1 = "/archparty";
	
	SlashCmdList["ARCHPET"] = Archaeologist_Slash_ArchPet;
    SLASH_ARCHPET1 = "/archpet";
	
	SlashCmdList["ARCHTARGET"] = Archaeologist_Slash_ArchTarget;
    SLASH_ARCHTARGET1 = "/archtarget";
	
	SlashCmdList["ARCHOPTIONS"] = Archaeologist_Slash_Arch;
    SLASH_ARCHOPTIONS1 = "/arch";

end



-- <= == == == == == == == == == == == == =>
-- => Slash Commands
-- <= == == == == == == == == == == == == =>

function Archaeologist_Slash_ArchHelp()

	local color = {r = 0, g = 1, b = 0.2};
	Sea.io.printc(color,"Archaeologist Help");
	Sea.io.printc(color,"----------------------------------------------");
	Sea.io.printc(color,"*** Player Options ***");
	Sea.io.printc(color,"/archplayer text [hp/mp] [on/off/toggle]");
	Sea.io.printc(color,"/archplayer percent [hp/mp/xp] [on/off/toggle]");
	Sea.io.printc(color,"/archplayer prefix [hp/mp/xp] [on/off/toggle]");
	Sea.io.printc(color,"*** Party Options ***");
	Sea.io.printc(color,"/archparty text [hp/mp] [on/off/toggle]");
	Sea.io.printc(color,"/archparty percent [hp/mp] [on/off/toggle]");
	Sea.io.printc(color,"/archparty prefix [hp/mp] [on/off/toggle]");
	Sea.io.printc(color,"/archparty [buffs/debuffs] [on/off/toggle/#]");
	Sea.io.printc(color,"*** Pet Options ***");
	Sea.io.printc(color,"/archpet text [hp/mp] [on/off/toggle]");
	Sea.io.printc(color,"/archpet percent [hp/mp] [on/off/toggle]");
	Sea.io.printc(color,"/archpet prefix [hp/mp] [on/off/toggle]");
	Sea.io.printc(color,"/archpet [buffs/debuffs] [on/off/toggle/#]");
	Sea.io.printc(color,"*** Target Options ***");
	Sea.io.printc(color,"/archtarget text [hp/mp] [on/off/toggle]");
	Sea.io.printc(color,"/archtarget percent [hp/mp] [on/off/toggle]");
	Sea.io.printc(color,"/archtarget prefix [hp/mp] [on/off/toggle]");
	Sea.io.printc(color,"/archtarget [buffs/debuffs] [on/off/toggle/#]");
	Sea.io.printc(color,"*** Special Options ***");
	Sea.io.printc(color,"/arch [hpcolor/debuffalt/hpmpalt] [on/off/toggle]");
	Sea.io.printc(color,"/arch [textsize1/textsize2] [#]");
	Sea.io.printc(color,"(textsize1: player, target. textsize2: pet, party.  #: 6-20)");

end

function Archaeologist_Slash_Toggle(var, onVal, status)
	if (var) and (onVal) and (status) then
		--Sea.io.print(var.." "..status.."("..onVal..")");
		if (ArchaeologistVarData[var].func) then
			if (status == "ON") then
				ArchaeologistVarData[var].func(onVal);
				ArchaeologistVars[var] = onVal;
			elseif (status == "OFF") then
				ArchaeologistVarData[var].func(BinaryInvert(onVal));
				ArchaeologistVars[var] = BinaryInvert(onVal);
			elseif (status == "TOGGLE") then
				local val = BinaryInvert(ArchaeologistVars[var])
				ArchaeologistVarData[var].func(val);
				ArchaeologistVars[var] = val;
			else
				Archaeologist_Slash_ArchHelp();
				return;
			end
		else
			--some internal error with the function not being found
			Sea.io.print("Error: Toggle Function Not Found");
			return;
		end
		local readValue = "OFF";
		if (ArchaeologistVars[var] == onVal) then
			readValue = "ON";
		end
		Sea.io.print(getglobal("ARCHAEOLOGIST_CONFIG_"..var).." "..readValue);
		Archaeologist_VarSync_VarsToUltimateUI();
	end
end

function Archaeologist_Slash_SetSlider(var, number)
	if (var) and (number) then
		number = tonumber(number);
		if (number < ArchaeologistVarData[var].min) then
			number = ArchaeologistVarData[var].min;
		elseif (number > ArchaeologistVarData[var].max) then
			number = ArchaeologistVarData[var].max;
		end
		
		ArchaeologistVarData[var].func(1, number);
		Sea.io.print(getglobal("ARCHAEOLOGIST_CONFIG_"..var)," ",number);
		Archaeologist_VarSync_VarsToUltimateUI();
	else
		Archaeologist_Slash_ArchHelp();
	end
end


function Archaeologist_Slash_ArchPlayer(msg)
	local var;
	local onVal;
	msg = string.upper(msg);
	local startpos, endpos, option, type, status = string.find(msg, "(%w+) (%w+) (%w+)");
	if (option) and (type) and (status) then
	
		if (option == "TEXT") then
			onVal = 1;
			if (type == "HP") then
				var = "PLAYERHP";
			elseif (type == "MP") then
				var = "PLAYERMP";
			elseif (type == "XP") then
				var = "PLAYERXP";
			end
		
		elseif (option == "PERCENT") then
			onVal = 1;
			if (type == "HP") then
				var = "PLAYERHPP";
			elseif (type == "MP") then
				var = "PLAYERMPP";
			elseif (type == "XP") then
				var = "PLAYERXPP";
			end
			
		elseif (option == "PREFIX") then
			onVal = 0;
			if (type == "HP") then
				var = "PLAYERHPJUSTVALUE";
			elseif (type == "MP") then
				var = "PLAYERMPJUSTVALUE";
			elseif (type == "XP") then
				var = "PLAYERXPJUSTVALUE";
			end
			
		end
		
	end
	if (var) and (onVal) and (status) then
		Archaeologist_Slash_Toggle(var, onVal, status);
	else
		Archaeologist_Slash_ArchHelp();
	end
end


function Archaeologist_Slash_ArchParty(msg)
	local var;
	local onVal;
	msg = string.upper(msg);
	local startpos, endpos, option, type, status = string.find(msg, "(%w+) (%w+) (%w+)");
	if (option) and (type) and (status) then
	
		if (option == "TEXT") then
			onVal = 1;
			if (type == "HP") then
				var = "PARTYHP";
			elseif (type == "MP") then
				var = "PARTYMP";
			end
		
		elseif (option == "PERCENT") then
			onVal = 1;
			if (type == "HP") then
				var = "PARTYHPP";
			elseif (type == "MP") then
				var = "PARTYMPP";
			end
			
		elseif (option == "PREFIX") then
			onVal = 0;
			if (type == "HP") then
				var = "PARTYHPJUSTVALUE";
			elseif (type == "MP") then
				var = "PARTYMPJUSTVALUE";
			end
			
		end
	else
		startpos, endpos, option, number = string.find(msg, "(%w+) (%d+)");
		if (option) and (number) then
		
			if (option == "BUFFS") then
				var = "PBUFFNUM";
			elseif (option == "DEBUFFS") then
				var = "PDEBUFFNUM";
			end
			Archaeologist_Slash_SetSlider(var, number);
			return;
			
		else
			startpos, endpos, option, status = string.find(msg, "(%w+) (%w+)");
			if (option) and (status) then
			
				if (option == "BUFFS") then
					onVal = 0;
					var = "PBUFFS";
					
				elseif (option == "DEBUFFS") then
					onVal = 0;
					var = "PDEBUFFS";
					
				end
			end
		end
	end
	if (var) and (onVal) and (status) then
		Archaeologist_Slash_Toggle(var, onVal, status);
	else
		Archaeologist_Slash_ArchHelp();
	end
end


function Archaeologist_Slash_ArchPet(msg)
	local var;
	local onVal;
	msg = string.upper(msg);
	local startpos, endpos, option, type, status = string.find(msg, "(%w+) (%w+) (%w+)");
	if (option) and (type) and (status) then
	
		if (option == "TEXT") then
			onVal = 1;
			if (type == "HP") then
				var = "PETHP";
			elseif (type == "MP") then
				var = "PETMP";
			end
		
		elseif (option == "PERCENT") then
			onVal = 1;
			if (type == "HP") then
				var = "PETHPP";
			elseif (type == "MP") then
				var = "PETMPP";
			end
			
		elseif (option == "PREFIX") then
			onVal = 0;
			if (type == "HP") then
				var = "PETHPJUSTVALUE";
			elseif (type == "MP") then
				var = "PETMPJUSTVALUE";
			end
			
		end
	
	else
		startpos, endpos, option, number = string.find(msg, "(%w+) (%d+)");
		if (option) and (number) then
		
			if (option == "BUFFS") then
				var = "PTBUFFNUM";
			elseif (option == "DEBUFFS") then
				var = "PTDEBUFFNUM";
			end
			Archaeologist_Slash_SetSlider(var, number);
			return;
			
		else
			startpos, endpos, option, status = string.find(msg, "(%w+) (%w+)");
			if (option) and (status) then
			
				if (option == "BUFFS") then
					onVal = 0;
					var = "PTBUFFS";
					
				elseif (option == "DEBUFFS") then
					onVal = 0;
					var = "PTDEBUFFS";
					
				end
			end
		end
	end
	if (var) and (onVal) and (status) then
		Archaeologist_Slash_Toggle(var, onVal, status);
	else
		Archaeologist_Slash_ArchHelp();
	end
end


function Archaeologist_Slash_ArchTarget(msg)
	local var;
	local onVal;
	msg = string.upper(msg);
	local startpos, endpos, option, type, status = string.find(msg, "(%w+) (%w+) (%w+)");
	if (option) and (type) and (status) then
	
		if (option == "TEXT") then
			onVal = 1;
			if (type == "HP") then
				var = "TARGETHP";
			elseif (type == "MP") then
				var = "TARGETMP";
			end
		
		elseif (option == "PERCENT") then
			onVal = 1;
			if (type == "HP") then
				var = "TARGETHPP";
			elseif (type == "MP") then
				var = "TARGETMPP";
			end
			
		elseif (option == "PREFIX") then
			onVal = 0;
			if (type == "HP") then
				var = "TARGETHPJUSTVALUE";
			elseif (type == "MP") then
				var = "TARGETMPJUSTVALUE";
			end
			
		end
	
	else
		startpos, endpos, option, number = string.find(msg, "(%w+) (%d+)");
		if (option) and (number) then
		
			if (option == "BUFFS") then
				var = "TBUFFNUM";
			elseif (option == "DEBUFFS") then
				var = "TDEBUFFNUM";
			end
			Archaeologist_Slash_SetSlider(var, number);
			return;
			
		else
			startpos, endpos, option, status = string.find(msg, "(%w+) (%w+)");
			if (option) and (status) then
			
				if (option == "BUFFS") then
					onVal = 0;
					var = "TBUFFS";
					
				elseif (option == "DEBUFFS") then
					onVal = 0;
					var = "TDEBUFFS";
					
				end
			end
		end
		
	end
	if (var) and (onVal) and (status) then
		Archaeologist_Slash_Toggle(var, onVal, status);
	else
		Archaeologist_Slash_ArchHelp();
	end
end


function Archaeologist_Slash_Arch(msg)
	local var;
	local onVal;
	msg = string.upper(msg);
	local startpos, endpos, option, status = string.find(msg, "(%w+) (%w+)");
	if (option) and (type) and (status) then
	
		if (option == "HPCOLOR") then
			onVal = 1;
			var = "HPCOLOR";
		
		elseif (option == "DEBUFFALT") then
			onVal = 1;
			var = "DEBUFFALT";
			
		elseif (option == "HPMPALT") then
			onVal = 1;
			var = "HPMPALT";
		
		else
			startpos, endpos, option, number = string.find(msg, "(%w+) (%d+)");
			if (option) and (number) then
			
				if (option == "TEXTSIZE1") then
					var = "HPMPLARGESIZE";
				elseif (option == "TEXTSIZE2") then
					var = "HPMPSMALLSIZE";
				end
				Archaeologist_Slash_SetSlider(var, number);
				return;
			end
		
		end
		
	end
	if (var) and (onVal) and (status) then
		Archaeologist_Slash_Toggle(var, onVal, status);
	else
		Archaeologist_Slash_ArchHelp();
	end
end


