--[[
	XUI_BagButtons.lua
	XUIVersion++2004.11.15++

	Capsule to allow movement of Bag buttons.
	
]]

-- ** DEFAULT SETTINGS **--
-- Default BagButton capsule show, off
XUI_DefaultShowBagButtons = 0;
-- Default local lock setting
XUI_DefaultLockBagButtonsCapsule = 0;

-- ** GENERAL VARIABLES ** --
-- Show Bag buttons binding name
BINDING_NAME_XUISHOWBAGBUTTONS = "Toggle Bag Buttons";

-- ** Bag BUTTON CAPSULE FUNCTIONS ** --
-- Event registers
function XUI_BagButtonCapsuleOnLoad ()
	this:RegisterEvent("VARIABLES_LOADED");
end

-- Registrations
function XUI_BagButtonCapsuleOnEvent (event)
	if (event == "VARIABLES_LOADED") then
		if (XUI_SHELL == 1) then
			if (XUI_RetrieveSaved("ShowBagButtons") == nil) then
				XUI_ShowBagButtons = XUI_DefaultShowBagButtons;
			else
				XUI_ShowBagButtons = XUI_RetrieveSaved("ShowBagButtons");
			end
			XUI_RegisterOption(121, "check", XUI_ShowBagButtons, "ShowBagButtons", XUI_UpdateBagButtons, "Show Bag Buttons", "Shows the default action bar Bag buttons\nin their own movable capsule.\nNote this option can be duplicated with a key\nbinding to quickly show or hide the buttons.");
		else
			if (XUI_ShowBagButtons == nil) then
				XUI_ShowBagButtons = XUI_DefaultShowBagButtons;
			end
			--RegisterForSave("XUI_ShowBagButtons");
			SlashCmdList["XUI_SHOWBagBUTTONS"] = XUI_ShowBagButtonsSlashHandler;
			SLASH_XUI_SHOWBagBUTTONS1 = "/hotbarshowBagbuttons";
			SLASH_XUI_SHOWBagBUTTONS2 = "/hbshowBagbuttons";
			SlashCmdList["XUI_LOCKBagBUTTONS"] = XUI_LockBagButtonsSlashHandler;
			SLASH_XUI_LOCKBagBUTTONS1 = "/hotbarlockBagbuttons";
			SLASH_XUI_LOCKBagBUTTONS2 = "/hblockBagbuttons";
		end
		return;
	end
end	

-- Set the capsule backdrop if it shows
function XUI_BagButtonCapsuleArtOnShow ()
	this:SetBackdropBorderColor(0, 0, 0);
	this:SetBackdropColor(0, 0, 0);
end

-- Function to show and hide the Bag buttons when the key binding to do so is pressed
function XUI_ShowBagExecuteBinding ()
	if (XUI_RetrieveOption ~= nil) then
		if (XUI_RetrieveOption(121) ~= nil) then
			XUI_ShowBagButtons = XUI_RetrieveOption(121)[XUI_VALUE];
		end
	end
	-- Simply toggle
	if (XUI_ShowBagButtons == 1) then
		XUI_ShowBagButtons = 0;
	else
		XUI_ShowBagButtons = 1;
	end
	-- Update setting in Shell
	if (XUI_SHELL == 1) then
		XUI_UpdateValue(121, XUI_ShowBagButtons);
	end
	-- Update function
	XUI_UpdateBagButtons();
end

-- Capsule watch
function XUI_BagButtonCapsuleOnUpdate ()
	XUI_ToggleBagButtonCapsule();
end

-- Capsule toggle
function XUI_ToggleBagButtonCapsule ()
	if (XUI_RetrieveOption ~= nil) then
		if (XUI_RetrieveOption(121) ~= nil) then
			XUI_ShowBagButtons = XUI_RetrieveOption(121)[XUI_VALUE];
		end
	end
	if (XUI_ShowBagButtons == 1) then
		-- If Shell is present, update the local lock setting
		if (XUI_SHELL ~= nil) then
			XUI_LockBagButtonsCapsule = XUI_RetrieveOption(102)[XUI_VALUE];
		end	
		-- Then, if the action bar is not locked, go ahead and show/hide or drag the capsule as needed, or if it is locked, hide the capsule so it can't be dragged
		if (XUI_LockBagButtonsCapsule == 0) then
			-- Make sure the capsule is showing, it might have been previously hidden
			if (not XUI_BagButtonCapsule:IsVisible()) then
				XUI_BagButtonCapsule:Show();
			end
			-- Quick function to check if the mouse is anywhere over our capsule
			if (MouseIsOver(XUI_BagButtonCapsule)) then
				XUI_BagButtonCapsuleArt:Show();
			else
				XUI_BagButtonCapsuleArt:Hide();
			end
		else
			XUI_BagButtonCapsule:Hide();
		end
	else
		XUI_BagButtonCapsule:Hide();
	end
end

-- Move/Show Bag buttons as we need
function XUI_UpdateBagButtons ()
	if (XUI_RetrieveOption ~= nil) then
		if (XUI_RetrieveOption(101) ~= nil) then
			XUI_EnableHotBar = XUI_RetrieveOption(101)[XUI_VALUE];
		end
		if (XUI_RetrieveOption(121) ~= nil) then
			XUI_ShowBagButtons = XUI_RetrieveOption(121)[XUI_VALUE];
		end
	end
	-- Move the main backpack button depending on the gapping
	local gap = XUI_ActionButtonGapMax - XUI_ActionButtonGap;
	local negativeGap = gap - (gap * 2);
	local bagGap = gap - 7;
	MainMenuBarBackpackButton:ClearAllPoints();
	-- If the HotBar is enabled, determine if we're showing Bag buttons or not. Else show them in their default spots, and hide the capsule.
	if (XUI_EnableHotBar == 1) then
		-- Show Bag buttons, reroute the talent button function, and anchor to the capsule
		if (XUI_ShowBagButtons == 1) then
			-- move bag stuff to the capsule with correct gapping
			XUI_BagButtonCapsule:Show();
			MainMenuBarBackpackButton:SetPoint("TOPRIGHT", "XUI_BagButtonCapsule", "TOPRIGHT", -7, -20);
		else
			-- Hide Bags so they dont show and hide the bag capsule
			XUI_BagButtonCapsule:Hide();
			MainMenuBarBackpackButton:SetPoint("BOTTOMRIGHT", "MainMenuBarArtFrame", "BOTTOMRIGHT", -6, 2);
			return;
		end
	else
		-- Hide bag capsule AND Move bag stuff back to the original location with original gapping
		XUI_BagButtonCapsule:Hide();
		MainMenuBarBackpackButton:SetPoint("BOTTOMRIGHT", "MainMenuBarArtFrame", "BOTTOMRIGHT", -6, 2);
	end
	-- Move the remaining bag buttons anchored to the backpack button
	for i=0, 3 do
		-- Get current button
		local button = getglobal("CharacterBag"..i.."Slot");
		-- Decrement the ID
		local lastID = i-1;
		-- Get the last button
		local lastButton = "CharacterBag"..lastID.."Slot";
		-- Convert our gap number to negative, since we're going the other way
		local negativeGap = XUI_ActionButtonGap - (XUI_ActionButtonGap * 2);
		-- Add one because the bag buttons are slightly different than action buttons
		local bagGap = negativeGap + 1;
		button:ClearAllPoints();
		-- If this is the first, anchor it to the backpack button, else anchor it to the last button
		if (i == 0) then
			button:SetPoint("RIGHT", "MainMenuBarBackpackButton", "LEFT", bagGap, 0);
		else
			button:SetPoint("RIGHT", lastButton, "LEFT", bagGap, 0);
		end
	end
end

-- ** SLASH HANDLERS ** --
function XUI_ShowBagButtonsSlashHandler (msg)
	-- Due to the nature of slash commands, 'msg' will never be nil, so we don't require a check
	msg = string.lower(msg);
	if (msg == "show" or msg == "1" or msg == "true") then
		XUI_ShowBagButtons = 1;
		DEFAULT_CHAT_FRAME:AddMessage("Showing Bag buttons & capsule.", 1, 1, 1);
	elseif (msg == "hide" or msg == "0" or msg == "false") then
		XUI_ShowBagButtons= 0;
		DEFAULT_CHAT_FRAME:AddMessage("Hiding Bag buttons & capsule.", 1, 1, 1);
	elseif (msg == "default" or msg == "reset" or msg == "revert") then
		XUI_ShowBagButtons = XUI_DefaultShowBagButtons;
		DEFAULT_CHAT_FRAME:AddMessage("Reverting Bag button display state to default.", 1, 1, 1);
	elseif (msg == "help") then
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /hotbarshowBagbuttons /hbshowBagbuttons", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   show, true, or 1 - Show Bag buttons & capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   hide, false, or 0 - Hide Bag buttons & capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default state.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   blank - Toggle based on current state.", 1, 1, 1);
	elseif (msg == "") then
		if (XUI_ShowBagButtons == 1) then 
			XUI_ShowBagButtons = 0; 
			DEFAULT_CHAT_FRAME:AddMessage("Hiding Bag buttons & capsule.", 1, 1, 1);
		else 
			XUI_ShowBagButtons = 1; 
			DEFAULT_CHAT_FRAME:AddMessage("Showing Bag buttons & capsule.", 1, 1, 1);
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /hotbarshowBagbuttons /hbshowBagbuttons", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   show, true, or 1 - Show Bag buttons & capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   hide, false, or 0 - Hide Bag buttons & capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default state.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   blank - Toggle based on current state.", 1, 1, 1);
	end
	XUI_UpdateBagButtons();
end

function XUI_LockBagButtonsSlashHandler (msg)
	-- Due to the nature of slash commands, 'msg' will never be nil, so we don't require a check
	msg = string.lower(msg);
	if (msg == "yes" or msg == "1" or msg == "true") then
		XUI_LockBagButtonsCapsule = 1;
		DEFAULT_CHAT_FRAME:AddMessage("Locking Bag buttons capsule.", 1, 1, 1);
	elseif (msg == "no" or msg == "0" or msg == "false") then
		XUI_LockBagButtonsCapsule= 0;
		DEFAULT_CHAT_FRAME:AddMessage("Unlocking Bag buttons capsule.", 1, 1, 1);
	elseif (msg == "default" or msg == "reset" or msg == "revert") then
		XUI_LockBagButtonsCapsule = XUI_DefaultLockBagButtonsCapsule;
		DEFAULT_CHAT_FRAME:AddMessage("Reverting Bag button capsule lock state to default.", 1, 1, 1);
	elseif (msg == "help") then
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /hotbarlockBagbuttons /hblockBagbuttons", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   yes, true, or 1 - Lock Bag buttons capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   no, false, or 0 - Unlock Bag buttons capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default state.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   blank - Toggle based on current state.", 1, 1, 1);
	elseif (msg == "") then
		if (XUI_LockBagButtonsCapsule == 1) then 
			XUI_LockBagButtonsCapsule = 0; 
			DEFAULT_CHAT_FRAME:AddMessage("Unlocking Bag buttons capsule.", 1, 1, 1);
		else 
			XUI_LockBagButtonsCapsule = 1; 
			DEFAULT_CHAT_FRAME:AddMessage("Locking Bag buttons capsule.", 1, 1, 1);
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /hotbarlockBagbuttons /hblockBagbuttons", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   yes, true, or 1 - Lock Bag buttons capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   no, false, or 0 - Unlock Bag buttons capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default state.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   blank - Toggle based on current state.", 1, 1, 1);
	end
end
