--[[
	XUI_ClassButtons.lua
	XUIVersion++2004.11.15++

	Capsule to allow movement of Class buttons.
	
]]

-- ** DEFAULT SETTINGS **--
-- Default ClassButton capsule show, off
XUI_DefaultShowClassButtons = 0;
-- Default local lock setting
XUI_DefaultLockClassButtonsCapsule = 0;

-- ** GENERAL VARIABLES ** --
-- Show Class buttons binding name
BINDING_NAME_XUISHOWCLASSBUTTONS = "Toggle Class Buttons";

-- ** Class BUTTON CAPSULE FUNCTIONS ** --
-- Event registers
function XUI_ClassButtonCapsuleOnLoad ()
	this:RegisterEvent("VARIABLES_LOADED");
end

-- Registrations
function XUI_ClassButtonCapsuleOnEvent (event)
	if (event == "VARIABLES_LOADED") then
		if (XUI_SHELL == 1) then
			if (XUI_RetrieveSaved("ShowClassButtons") == nil) then
				XUI_ShowClassButtons = XUI_DefaultShowClassButtons;
			else
				XUI_ShowClassButtons = XUI_RetrieveSaved("ShowClassButtons");
			end
			XUI_RegisterOption(123, "check", XUI_ShowClassButtons, "ShowClassButtons", XUI_UpdateClassButtons, "Show Class Buttons", "Shows the default action bar Class buttons\nin their own movable capsule.\nNote this option can be duplicated with a key\nbinding to quickly show or hide the buttons.");
		else
			if (XUI_ShowClassButtons == nil) then
				XUI_ShowClassButtons = XUI_DefaultShowClassButtons;
			end
			--RegisterForSave("XUI_ShowClassButtons");
			SlashCmdList["XUI_SHOWClassBUTTONS"] = XUI_ShowClassButtonsSlashHandler;
			SLASH_XUI_SHOWClassBUTTONS1 = "/hotbarshowClassbuttons";
			SLASH_XUI_SHOWClassBUTTONS2 = "/hbshowClassbuttons";
			SlashCmdList["XUI_LOCKClassBUTTONS"] = XUI_LockClassButtonsSlashHandler;
			SLASH_XUI_LOCKClassBUTTONS1 = "/hotbarlockClassbuttons";
			SLASH_XUI_LOCKClassBUTTONS2 = "/hblockClassbuttons";
		end
		return;
	end
end	

-- Set the capsule backdrop if it shows
function XUI_ClassButtonCapsuleArtOnShow ()
	this:SetBackdropBorderColor(0, 0, 0);
	this:SetBackdropColor(0, 0, 0);
end

-- Function to show and hide the Class buttons when the key binding to do so is pressed
function XUI_ShowClassExecuteBinding ()
	if (XUI_RetrieveOption ~= nil) then
		if (XUI_RetrieveOption(123) ~= nil) then
			XUI_ShowClassButtons = XUI_RetrieveOption(123)[XUI_VALUE];
		end
	end
	-- Simply toggle
	if (XUI_ShowClassButtons == 1) then
		XUI_ShowClassButtons = 0;
	else
		XUI_ShowClassButtons = 1;
	end
	-- Update setting in Shell
	if (XUI_SHELL == 1) then
		XUI_UpdateValue(123, XUI_ShowClassButtons);
	end
	-- Update function
	XUI_UpdateClassButtons();
end

-- Capsule watch
function XUI_ClassButtonCapsuleOnUpdate ()
	XUI_ToggleClassButtonCapsule();
end

-- Capsule toggle
function XUI_ToggleClassButtonCapsule ()
	if (XUI_RetrieveOption ~= nil) then
		if (XUI_RetrieveOption(123) ~= nil) then
			XUI_ShowClassButtons = XUI_RetrieveOption(123)[XUI_VALUE];
		end
	end
	if (XUI_ShowClassButtons == 1) then
		-- If Shell is present, update the local lock setting
		if (XUI_SHELL ~= nil) then
			XUI_LockClassButtonsCapsule = XUI_RetrieveOption(102)[XUI_VALUE];
		end	
		-- Then, if the action bar is not locked, go ahead and show/hide or drag the capsule as needed, or if it is locked, hide the capsule so it can't be dragged
		if (XUI_LockClassButtonsCapsule == 0) then
			-- Make sure the capsule is showing, it might have been previously hidden
			if (not XUI_ClassButtonCapsule:IsVisible()) then
				XUI_ClassButtonCapsule:Show();
			end
			-- Quick function to check if the mouse is anywhere over our capsule
			if (MouseIsOver(XUI_ClassButtonCapsule)) then
				XUI_ClassButtonCapsuleArt:Show();
			else
				XUI_ClassButtonCapsuleArt:Hide();
			end
		else
			XUI_ClassButtonCapsule:Hide();
		end
	else
		XUI_ClassButtonCapsule:Hide();
	end
end

-- Move/Show Class buttons as we need
function XUI_UpdateClassButtons ()
	if (XUI_RetrieveOption ~= nil) then
		if (XUI_RetrieveOption(101) ~= nil) then
			XUI_EnableHotBar = XUI_RetrieveOption(101)[XUI_VALUE];
		end
		if (XUI_RetrieveOption(123) ~= nil) then
			XUI_ShowClassButtons = XUI_RetrieveOption(123)[XUI_VALUE];
		end
	end
	-- If the HotBar is enabled, determine if we're showing Class buttons or not. Else show them in their default spots, and hide the capsule.
	if (XUI_EnableHotBar == 1) then
		-- Show Class buttons, reroute the talent button function, and anchor to the capsule
		if (XUI_ShowClassButtons == 1) then
			-- Show class capsule and move class bar stuff to this capsule
			XUI_ClassButtonCapsule:Show();
			ShapeshiftBarFrame:ClearAllPoints();
			ShapeshiftBarFrame:SetPoint("TOPLEFT", "XUI_ClassButtonCapsule", "TOPLEFT", 0, -25);
		else
			-- Hide class buttons and class capsule
			XUI_ClassButtonCapsule:Hide();
			ShapeshiftBarFrame:ClearAllPoints();
			ShapeshiftBarFrame:SetPoint("BOTTOMLEFT", "MainMenuBar", "TOPLEFT", 30, 0);
			return;
		end
	else
		-- Hide the class capsule, reset buttons to default position
		XUI_ClassButtonCapsule:Hide();
		ShapeshiftBarFrame:ClearAllPoints();
		ShapeshiftBarFrame:SetPoint("BOTTOMLEFT", "MainMenuBar", "TOPLEFT", 30, 0);
		ShapeshiftBar_UpdatePosition();
	end
end

-- ** SLASH HANDLERS ** --
function XUI_ShowClassButtonsSlashHandler (msg)
	-- Due to the nature of slash commands, 'msg' will never be nil, so we don't require a check
	msg = string.lower(msg);
	if (msg == "show" or msg == "1" or msg == "true") then
		XUI_ShowClassButtons = 1;
		DEFAULT_CHAT_FRAME:AddMessage("Showing Class buttons & capsule.", 1, 1, 1);
	elseif (msg == "hide" or msg == "0" or msg == "false") then
		XUI_ShowClassButtons= 0;
		DEFAULT_CHAT_FRAME:AddMessage("Hiding Class buttons & capsule.", 1, 1, 1);
	elseif (msg == "default" or msg == "reset" or msg == "revert") then
		XUI_ShowClassButtons = XUI_DefaultShowClassButtons;
		DEFAULT_CHAT_FRAME:AddMessage("Reverting Class button display state to default.", 1, 1, 1);
	elseif (msg == "help") then
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /hotbarshowClassbuttons /hbshowClassbuttons", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   show, true, or 1 - Show Class buttons & capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   hide, false, or 0 - Hide Class buttons & capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default state.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   blank - Toggle based on current state.", 1, 1, 1);
	elseif (msg == "") then
		if (XUI_ShowClassButtons == 1) then 
			XUI_ShowClassButtons = 0; 
			DEFAULT_CHAT_FRAME:AddMessage("Hiding Class buttons & capsule.", 1, 1, 1);
		else 
			XUI_ShowClassButtons = 1; 
			DEFAULT_CHAT_FRAME:AddMessage("Showing Class buttons & capsule.", 1, 1, 1);
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /hotbarshowClassbuttons /hbshowClassbuttons", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   show, true, or 1 - Show Class buttons & capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   hide, false, or 0 - Hide Class buttons & capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default state.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   blank - Toggle based on current state.", 1, 1, 1);
	end
	XUI_UpdateClassButtons();
end

function XUI_LockClassButtonsSlashHandler (msg)
	-- Due to the nature of slash commands, 'msg' will never be nil, so we don't require a check
	msg = string.lower(msg);
	if (msg == "yes" or msg == "1" or msg == "true") then
		XUI_LockClassButtonsCapsule = 1;
		DEFAULT_CHAT_FRAME:AddMessage("Locking Class buttons capsule.", 1, 1, 1);
	elseif (msg == "no" or msg == "0" or msg == "false") then
		XUI_LockClassButtonsCapsule= 0;
		DEFAULT_CHAT_FRAME:AddMessage("Unlocking Class buttons capsule.", 1, 1, 1);
	elseif (msg == "default" or msg == "reset" or msg == "revert") then
		XUI_LockClassButtonsCapsule = XUI_DefaultLockClassButtonsCapsule;
		DEFAULT_CHAT_FRAME:AddMessage("Reverting Class button capsule lock state to default.", 1, 1, 1);
	elseif (msg == "help") then
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /hotbarlockClassbuttons /hblockClassbuttons", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   yes, true, or 1 - Lock Class buttons capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   no, false, or 0 - Unlock Class buttons capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default state.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   blank - Toggle based on current state.", 1, 1, 1);
	elseif (msg == "") then
		if (XUI_LockClassButtonsCapsule == 1) then 
			XUI_LockClassButtonsCapsule = 0; 
			DEFAULT_CHAT_FRAME:AddMessage("Unlocking Class buttons capsule.", 1, 1, 1);
		else 
			XUI_LockClassButtonsCapsule = 1; 
			DEFAULT_CHAT_FRAME:AddMessage("Locking Class buttons capsule.", 1, 1, 1);
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /hotbarlockClassbuttons /hblockClassbuttons", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   yes, true, or 1 - Lock Class buttons capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   no, false, or 0 - Unlock Class buttons capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default state.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   blank - Toggle based on current state.", 1, 1, 1);
	end
end
