--[[
	XUI_HotBar.lua
	XUIVersion++2004.11.14++
	
	This is our main action bar mod file.
	We've created an encapsulating frame
	to anchor the default action bar, bag
	buttons, and our own action buttons to.
	We can then allow for all of these to
	be dragged along with the capsule.
	
]]

-- ** DEFAULT SETTINGS ** --
-- Master HotBar enable
XUI_DefaultEnableHotBar = 1;
-- XUI_LOCKALLDEFAULT
XUI_DefaultLOCKALL = 1;
-- Default action bar specific lock setting
XUI_DefaultLockActionBar = 0;
-- Default number of pixels between action buttons
XUI_OriginalActionButtonGap = 6;
-- Default number of pixels between action buttons
XUI_DefaultActionButtonGap = 3;
-- Default maximum number of pixels between action buttons
XUI_DefaultActionButtonGapMax = 8;
-- Default minimum number of pixels between action buttons
XUI_DefaultActionButtonGapMin = 2;
-- Default stepping between each gap setting
XUI_DefaultActionButtonGapStep = 1;
-- Default setting for cast bar capsule lock
XUI_DefaultLockCastCapsule = 1;
-- Show lag meter by default
XUI_DefaultShowLagMeter = 1;
-- Enable casting bar capsule by default
XUI_DefaultAnchorCastingFrame = 1;

-- ** GENERAL VARIABLES ** --
-- Binding text definitions, HotBar header and page-on-press binding
BINDING_HEADER_XUIHOTBAR = "XUIMod HotBar";
BINDING_NAME_XUITOGGLELAGMETER = "Toggle Lag Meter";
-- Initial setup of action button gap slider settings
XUI_ActionButtonGapMax = XUI_DefaultActionButtonGapMax;
XUI_ActionButtonGapMin = XUI_DefaultActionButtonGapMin;
XUI_ActionButtonGapStep = XUI_DefaultActionButtonGapStep;

-- ** HOTBAR FUNCTIONS ** --
-- Event registration
function XUI_HotBarOnLoad ()
	this:RegisterEvent("VARIABLES_LOADED");
end

-- Configuration registrations
function XUI_HotBarOnEvent (event)
	if (event == "VARIABLES_LOADED") then
		if (XUI_RetrieveSaved("EnableHotBar") == nil) then
			XUI_EnableHotBar = XUI_DefaultEnableHotBar;
		else
			XUI_EnableHotBar = XUI_RetrieveSaved("EnableHotBar");
		end
		if (XUI_RetrieveSaved("XUI_LOCKALL") == nil) then
			XUI_LOCKALL = XUI_DefaultLOCKALL;
		else
			XUI_LOCKALL = XUI_RetrieveSaved("XUI_LOCKALL");
		end
		if (XUI_RetrieveSaved("ActionButtonGap") == nil) then
			XUI_ActionButtonGap = XUI_DefaultActionButtonGap;	
		else
			XUI_ActionButtonGap = XUI_RetrieveSaved("ActionButtonGap");
		end
		XUI_RegisterOption(100, "category", nil, nil, nil, "HotBar", "XUI Bar Toggles and Button Gapping.");
		XUI_RegisterOption(101, "check", XUI_EnableHotBar, "EnableHotBar", XUI_UpdateHotBar, "Enable HotBar", "Toggle between the XUI and default action bars.");
		XUI_RegisterOption(102, "check", XUI_LOCKALL, "XUI_LOCKALL", XUI_ActionButtonDisplayUpdate, "Lock all capsules.", "Lock all XUI bar capsules.");
		XUI_RegisterOption(104, "slider", XUI_ActionButtonGap, "ActionButtonGap", XUI_ActionButtonDisplayUpdate,"Action Button Gap", "Space, in pixels, between all action buttons,\nboth horizontally and vertically.", XUI_ActionButtonGapMin, XUI_ActionButtonGapMax, XUI_ActionButtonGapStep, " ");	
		XUI_RegisterOption(105, "button", nil, nil, XUI_ActionBarReset, "Reset", "Click to restore default settings.");
		XUI_UpdateHotBar();
	end
end

local old_ShapeshiftBar_UpdatePosition = ShapeshiftBar_UpdatePosition;
local function blankFunction ()
end

local old_PetActionBar_UpdatePosition = PetActionBar_UpdatePosition;
local function new_PetActionBar_UpdatePosition ()
	old_PetActionBar_UpdatePosition();
	if (SHOW_MULTI_ACTIONBAR_1 and SHOW_MULTI_ACTIONBAR_2) then
		PETACTIONBAR_YPOS = 130 + 72 + (XUI_ActionButtonGap * 3);
	elseif (SHOW_MULTI_ACTIONBAR_1 or SHOW_MULTI_ACTIONBAR_2) then
		PETACTIONBAR_YPOS = 130 + 36 + (XUI_ActionButtonGap * 2);
	else
		PETACTIONBAR_YPOS = 130;
	end
	SlidingActionBarTexture0:Hide();
	SlidingActionBarTexture1:Hide();
	PetActionBarFrame:SetPoint("TOPLEFT", "MainMenuBar", "BOTTOMLEFT", PETACTIONBAR_XPOS, PETACTIONBAR_YPOS);
end

local originalMultiActionBar_Update = MultiActionBar_Update;
function MultiActionBar_Update ()
	-- Hooking the default multibar update function so it can run our HotBar update function and keep things tidy
	originalMultiActionBar_Update();
	XUI_UpdateHotBar();
end

function XUI_UpdateHotBar (init)
	-- Take care of function calls without arguments
	if (init == nil) then
		init = 0;
	end
	-- Get XUIMod array value if available
	if (XUI_RetrieveOption ~= nil) then
		if (XUI_RetrieveOption(101) ~= nil) then
			XUI_EnableHotBar = XUI_RetrieveOption(101)[XUI_VALUE];
		end
	end
	-- Many changes to go between the HotBar and default main menu bar
	if (XUI_EnableHotBar == 1) then
		-- Need to get the bonus action bar out of the way
		BonusActionBarFrame:SetWidth(1);
		-- Losing unwanted graphical items, starting with the action bar page up/down buttons
		ActionBarUpButton:Hide();
		ActionBarDownButton:Hide();
		-- Four textures for the main menu bar background and 1 each for the endcaps
		MainMenuBarTexture0:Hide();
		MainMenuBarTexture1:Hide();
		MainMenuBarTexture2:Hide();
		MainMenuBarTexture3:Hide();
		MainMenuBarLeftEndCap:Hide();
		MainMenuBarRightEndCap:Hide();
		-- Not certain what this overlay is, didn't bother checking
		-- MainMenuBarOverlayFrame:Hide();
		-- Lag meter
		MainMenuBarPerformanceBarFrame:Hide();
		-- The experience status bar resisted hiding, so simply move it off the screen
		MainMenuExpBar:ClearAllPoints();
		MainMenuExpBar:SetPoint("BOTTOM", "UIParent", "BOTTOM", 0, -100);
		-- Rested tick
		ExhaustionTick:Hide();
		-- Main menu micro buttons
		XUI_UpdateMicroButtons();
		-- Bag buttons
		XUI_UpdateBagButtons();
		-- Pet buttons
		XUI_UpdatePetButtons();
		-- Class buttons
		XUI_UpdateClassButtons();
		-- cant hide so slide it off screen along with the weird duplicate
		BonusActionBarTexture0:ClearAllPoints();
		BonusActionBarTexture0:SetPoint("BOTTOM", "UIParent", "BOTTOM", 0, -100);
		-- Shapeshift/Stance/Aura background textures
		ShapeshiftBarLeft:Hide();
		ShapeshiftBarMiddle:Hide();
		ShapeshiftBarRight:Hide();	
		ShapeshiftBar_UpdatePosition = blankFunction;
		PetActionBar_UpdatePosition = blankFunction;
		-- Anchor the main menu bar to our capsule for movement
		MainMenuBar:ClearAllPoints();
		MainMenuBar:SetPoint("BOTTOM", "UIParent", "BOTTOM", 0, -100);
		MainMenuBar:SetWidth(300);
		-- Enable all of our options frame stuff
		XUI_Option104Thumb:Show();
		XUI_Option104Text:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);	
		XUI_Option104Low:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
		XUI_Option104High:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
		XUI_Option120:Enable();
		XUI_Option120Text:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
		XUI_Option121:Enable();
		XUI_Option121Text:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
		XUI_Option122:Enable();
		XUI_Option122Text:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
		XUI_Option123:Enable();
		XUI_Option123Text:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
	else
		-- Restore bonus action bar size
		BonusActionBarFrame:SetWidth(505);
		-- Set the main menu bar to it's default position
		MainMenuBar:SetWidth(1024);
		MainMenuBar:ClearAllPoints();
		MainMenuBar:SetPoint("BOTTOM", "UIParent", "BOTTOM", 0, 0);
		-- Show action bar arrows
		ActionBarUpButton:Show();
		ActionBarDownButton:Show();
		-- Show main menu bar textures, and end cap textures
		MainMenuBarTexture0:Show();
		MainMenuBarTexture1:Show();
		MainMenuBarTexture2:Show();
		MainMenuBarTexture3:Show();
		MainMenuBarLeftEndCap:Show();
		MainMenuBarRightEndCap:Show();
		-- Show overlay frame
		-- MainMenuBarOverlayFrame:Show();
		-- Show default lag meter
		MainMenuBarPerformanceBarFrame:Show();
		-- Move the experience bar to it's default position
		MainMenuExpBar:ClearAllPoints();
		MainMenuExpBar:SetPoint("TOP", "MainMenuBar", "TOP", 0, 0);
		-- Update the exhaustion tick to get it to display correctly
		ExhaustionTick_Update();
		-- Show micro buttons
		XUI_UpdateMicroButtons();
		-- Show bag buttons
		XUI_UpdateBagButtons();
		-- Show pet buttons
		XUI_UpdatePetButtons();
		-- Show class buttons
		XUI_UpdateClassButtons();
		-- Move the bonus action bar texture to it's default positiong
		BonusActionBarTexture0:ClearAllPoints();
		BonusActionBarTexture0:SetPoint("TOPLEFT", "BonusActionBarFrame", "TOPLEFT", 0, 0);
		-- Show special action button art
		ShapeshiftBarLeft:Show();
		ShapeshiftBarMiddle:Show();
		ShapeshiftBarRight:Show();	
		ShapeshiftBar_UpdatePosition = old_ShapeshiftBar_UpdatePosition;
		ShapeshiftBar_UpdatePosition();
		-- Move pet art to it's default position
		SlidingActionBarTexture0:ClearAllPoints();
		SlidingActionBarTexture0:SetPoint("TOPLEFT", "PetActionBarFrame", "TOPLEFT", 0, 0);
		-- Move the special action button bar to it's default position
		ShapeshiftBarFrame:ClearAllPoints();
		ShapeshiftBarFrame:SetPoint("BOTTOMLEFT", "MainMenuBar", "TOPLEFT", 30, 0);
		-- Change the Y target of the pet action bar
		--PETACTIONBAR_YPOS = 98;
		PetActionBar_UpdatePosition = old_PetActionBar_UpdatePosition;
		-- If there is a pet bar visible, we have to update it to get it to effect our Y target change
		if (PetHasActionBar()) then	
			-- Run the update
			PetActionBar_Update();
			-- Unlock, hide, show, and relock to display changes
			UnlockPetActionBar();
			HidePetActionBar();			
			ShowPetActionBar();
			LockPetActionBar();
		end
		-- Disable XUIMod menu items
		XUI_Option104Thumb:Hide();
		XUI_Option104Text:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
		XUI_Option104Low:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
		XUI_Option104High:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
		XUI_Option120:Disable();
		XUI_Option120Text:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
		XUI_Option121:Disable();
		XUI_Option121Text:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
		XUI_Option122:Disable();
		XUI_Option122Text:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
		XUI_Option123:Disable();
		XUI_Option123Text:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
	end
	-- Update our action buttons display
	XUI_ActionButtonDisplayUpdate();
end

-- ** CASTING BAR FUNCTIONS ** --

-- Setup variable loading for the cast bar capsule
function XUI_CastBarCapsuleOnLoad ()
	this:RegisterEvent("VARIABLES_LOADED");
	this:RegisterEvent("SPELLCAST_START");
	this:RegisterEvent("SPELLCAST_STOP");
	this:RegisterEvent("SPELLCAST_FAILED");
	this:RegisterEvent("SPELLCAST_INTERRUPTED");
	this:RegisterEvent("SPELLCAST_DELAYED");
	this:RegisterEvent("SPELLCAST_CHANNEL_START");
	this:RegisterEvent("SPELLCAST_CHANNEL_UPDATE");
	XUI_CastBarCapsuleArt:SetBackdropBorderColor(0, 0, 0);
	XUI_CastBarCapsuleArt:SetBackdropColor(0, 0, 0);	
end

function XUI_CastBarCapsuleOnEvent (event)
	if (event == "VARIABLES_LOADED") then
		-- Set defaults if there is no saved value
		if (XUI_RetrieveSaved("AnchorCastingFrame") == nil) then
			XUI_AnchorCastingFrame = XUI_DefaultAnchorCastingFrame;
		else
			XUI_AnchorCastingFrame = XUI_RetrieveSaved("AnchorCastingFrame");
		end
		-- Register with XUIMod saving
		XUI_RegisterOption(52, "check", XUI_AnchorCastingFrame, "AnchorCastingFrame", XUI_UpdateCastFrameAnchor, "Enable Cast Bar Capsule", "Allows the dragging of the casting bar to\nany position on the screen, with lockability.");
		return;
	end
	if (event == "SPELLCAST_START" or event == "SPELLCAST_STOP" or event == "SPELLCAST_CHANNEL_START" or event == "SPELLCAST_CHANNEL_UPDATE" or event == "SPELLCAST_FAILED" or event == "SPELLCAST_DELAYED") then
		-- Watch for casting bar stuff to update our anchoring
		XUI_UpdateCastFrameAnchor();
		return;
	end
end

function XUI_CastBarCapsuleOnUpdate ()	
	-- Toggle the casting bar capsule, this will only occur if the UpdateCastFrameAnchor has determined to show the casting bar frame capsule
	XUI_ToggleCastBarCapsule();
	-- If the casting bar frame is visible then update our anchoring
	if (CastingBarFrame:IsVisible()) then
		XUI_UpdateCastFrameAnchor();
	end	
end

-- Update function for the casting frame anchoring
function XUI_UpdateCastFrameAnchor ()
	-- Retrieve XUIMod shell value if applicable
	if (XUI_RetrieveOption ~= nil) then
		if (XUI_RetrieveOption(52) ~= nil) then
			XUI_AnchorCastingFrame = XUI_RetrieveOption(52)[XUI_VALUE];
		end
	end
	-- If we're using the capsule, show it, and anchor the casting bar frame, else hide it and set the cast bar in it's default position
	if (XUI_AnchorCastingFrame == 1) then
		XUI_CastBarCapsule:Show();
		CastingBarFrame:ClearAllPoints();
		CastingBarFrame:SetPoint("BOTTOM", "XUI_CastBarCapsule", "BOTTOM", 0, 0);
		CastingBarFlash:ClearAllPoints();
		CastingBarFlash:SetPoint("CENTER", "XUI_CastBarCapsule", "CENTER", 0, -5);
		CastingBarFrameStatusBar:ClearAllPoints();
		CastingBarFrameStatusBar:SetPoint("CENTER", "XUI_CastBarCapsule", "CENTER", 0, -5);
		CastingBarText:ClearAllPoints();
		CastingBarText:SetPoint("CENTER", "XUI_CastBarCapsule", "CENTER", 0, -5);
	else
		XUI_CastBarCapsule:Hide();
		CastingBarFrame:ClearAllPoints();
		CastingBarFrame:SetPoint("BOTTOM", "UIParent", "BOTTOM", 0, 83);
		CastingBarFlash:ClearAllPoints();
		CastingBarFlash:SetPoint("CENTER", "CastingBarFrame", "CENTER", 0, 4);
		CastingBarFrameStatusBar:ClearAllPoints();
		CastingBarFrameStatusBar:SetPoint("CENTER", "CastingBarFrame", "CENTER", 0, 4);
		CastingBarText:ClearAllPoints();
		CastingBarText:SetPoint("TOP", "CastingBarFrame", "TOP", 0, 0);
	end
end

-- Casting bar capsule toggle function
function XUI_ToggleCastBarCapsule ()
	-- Get value from the Shell if enabled
	if (XUI_RetrieveOption ~= nil) then
		if (XUI_RetrieveOption(52) ~= nil) then
			XUI_AnchorCastingFrame = XUI_RetrieveOption(52)[XUI_VALUE];
		end
	end
	-- Only toggle if the capsule is enabled
	if (XUI_AnchorCastingFrame == 1) then
		-- If the Shell is present, update the local lock variable
		if (XUI_SHELL ~= nil) then
			XUI_LockCastCapsule = XUI_RetrieveOption(102)[XUI_VALUE];
		end
		-- Then, if XUIMod is not locked, go ahead and show/hide or drag the capsule as needed, or if it is locked, hide the capsule so it can't be dragged
		if (XUI_LockCastCapsule == 0) then
			if (not XUI_CastBarCapsule:IsVisible()) then
				XUI_CastBarCapsule:Show();
			end
			if (MouseIsOver(XUI_CastBarCapsule)) then
				XUI_CastBarCapsuleArt:Show();
			else
				XUI_CastBarCapsuleArt:Hide();
			end
		else
			XUI_CastBarCapsule:Hide();
		end
	end
end

-- ** LAG METER FUNCTIONS ** --

-- Lag meter onLoad, register for variable loading seperately
function XUI_LagMeterOnLoad()	
	this:RegisterEvent("VARIABLES_LOADED");
end

-- Take care of configuration things once variables are loaded
function XUI_LagMeterOnEvent (event)
	if (event == "VARIABLES_LOADED") then
		-- Set defaults if there is no saved value
		if (XUI_RetrieveSaved("ShowLagMeter") == nil) then
			XUI_ShowLagMeter = XUI_DefaultShowLagMeter;	
		else
			XUI_ShowLagMeter = XUI_RetrieveSaved("ShowLagMeter");
		end
		--Register with XUIMod saving
		XUI_RegisterOption(51, "check", XUI_ShowLagMeter, "ShowLagMeter", XUI_ToggleLagMeter, "Show Lag Meter", "Show or hide the XUIMod lag meter,\nlocated to the right of the minimap.");
		XUI_ToggleLagMeter();
		return;
	end
end

-- Lag meter toggle function
function XUI_ToggleLagMeter ()
	-- Check for registrations and update
	if (XUI_RetrieveOption ~= nil) then
		if (XUI_RetrieveOption(51) ~= nil) then
			XUI_ShowLagMeter = XUI_RetrieveOption(51)[XUI_VALUE];
		end
	end
	-- Show or hide
	if (XUI_ShowLagMeter == 1) then
		XUI_PerformanceBarFrame:Show();
		MiniMapMailFrame:ClearAllPoints();
		MiniMapMailFrame:SetPoint("BOTTOMRIGHT", "Minimap", "BOTTOMRIGHT", 18, -14);
		MiniMapMeetingStoneFrame:ClearAllPoints();
		MiniMapMeetingStoneFrame:SetPoint("TOP", "MiniMapMailFrame", "BOTTOM", -3, 5);
	else
		XUI_PerformanceBarFrame:Hide();
		MiniMapMailFrame:ClearAllPoints();
		MiniMapMailFrame:SetPoint("TOPRIGHT", "Minimap", "TOPRIGHT", 21, -38);
		MiniMapMeetingStoneFrame:ClearAllPoints();
		MiniMapMeetingStoneFrame:SetPoint("TOPRIGHT", "Minimap", "TOPRIGHT", 23, -65);
	end
end

function XUI_ActionButtonDisplayUpdate ()
	if (XUI_RetrieveOption ~= nil) then
		-- Update our local variables with updated settings from XUIMod if applicable	
		if (XUI_RetrieveOption(101) ~= nil) then
			XUI_EnableHotBar = XUI_RetrieveOption(101)[XUI_VALUE];
		end
		if (XUI_RetrieveOption(102) ~= nil) then
			if (XUI_RetrieveOption(102)[XUI_VALUE] ~= XUI_LOCKALL) then
				XUI_LOCKALL = XUI_RetrieveOption(102)[XUI_VALUE];
				XUI_FlexTLock();
			end	
		end
		if (XUI_RetrieveOption(104) ~= nil) then
			XUI_ActionButtonGap = XUI_RetrieveOption(104)[XUI_VALUE];
		end
	end
	-- Update the button gapping as well
	XUI_UpdateActionButtonGap();
end

function XUI_FlexTLock()
	if XUI_LOCKALL == 1 then
		XUI_LOCKALL = 0;
	else
		XUI_LOCKALL = 1;
	end
end

-- Reset all action bar options to default
function XUI_ActionBarReset()	
	-- Change settings in our arrays if the shell is present, else update the local variables
	XUI_UpdateValue(52, XUI_DefaultAnchorCastingFrame);
	XUI_UpdateValue(101, XUI_DefaultEnableHotBar);
	XUI_UpdateValue(102, XUI_DefaultLOCKALL);
	XUI_UpdateValue(104, XUI_DefaultActionButtonGap);
	XUI_UpdateValue(120, XUI_DefaultShowMicroButtons);
	XUI_UpdateValue(121, XUI_DefaultShowBagButtons);
	XUI_UpdateValue(122, XUI_DefaultShowPetButtons);
	XUI_UpdateValue(123, XUI_DefaultShowClassButtons);
	-- Master hotbar update function
	XUI_UpdateHotBar();
	-- Update the state of our check buttons if the shell is present
	XUI_Option101:SetChecked(XUI_DefaultEnableHotBar);
	XUI_Option102:SetChecked(XUI_DefaultLOCKALL);
	-- Set the action button gap slider to reflect changes
	XUI_Option104:SetValue(XUI_DefaultActionButtonGap);
	XUI_Option120:SetChecked(XUI_DefaultShowMicroButtons);
	XUI_Option121:SetChecked(XUI_DefaultShowBagButtons);
	XUI_Option122:SetChecked(XUI_DefaultShowPetButtons);
	XUI_Option123:SetChecked(XUI_DefaultShowClassButtons);
	-- Reset the hotbar position
	XUI_MicroButtonCapsule:ClearAllPoints();
	XUI_MicroButtonCapsule:SetPoint("BOTTOMRIGHT", "UIParent", "BOTTOMRIGHT", 6, -6);
	XUI_BagButtonCapsule:ClearAllPoints();
	XUI_BagButtonCapsule:SetPoint("BOTTOMRIGHT", "XUI_MicroButtonCapsule", "TOPRIGHT", 0, -30);
	XUI_PetButtonCapsule:ClearAllPoints();
	XUI_PetButtonCapsule:SetPoint("BOTTOMLEFT", "UIParent", "BOTTOMLEFT", -5, -5);
	XUI_ClassButtonCapsule:ClearAllPoints();
	XUI_ClassButtonCapsule:SetPoint("BOTTOM", "UIParent", "BOTTOM", 0, -5);
	XUI_CastBarCapsule:Show();
	XUI_CastBarCapsule:ClearAllPoints();
	XUI_CastBarCapsule:SetPoint("BOTTOM", "UIParent", "BOTTOM", 0, 100);
	XUI_UpdateCastFrameAnchor();
end

-- Move everything around that needs to be moved when we change action button gapping
function XUI_UpdateActionButtonGap()	
	-- If the HotBar is not enabled we have to use the default gap, prevents changing of the gap
	if (XUI_EnableHotBar == 0) then
		XUI_ActionButtonGap = XUI_OriginalActionButtonGap;
		XUI_UpdateValue(104, XUI_DefaultActionButtonGap);
	end	
	-- Change the Bar capsules size to adjust for gapping
	if (XUI_ActionButtonGap == 8) then
		XUI_BagButtonCapsule:SetWidth(228);
		XUI_BagButtonCapsuleArt:SetWidth(228);
		XUI_PetButtonCapsule:SetWidth(388);
		XUI_PetButtonCapsuleArt:SetWidth(388);
	elseif (XUI_ActionButtonGap == 7) then
		XUI_BagButtonCapsule:SetWidth(224);
		XUI_BagButtonCapsuleArt:SetWidth(224);
		XUI_PetButtonCapsule:SetWidth(379);
		XUI_PetButtonCapsuleArt:SetWidth(379);
	elseif (XUI_ActionButtonGap == 6) then
		XUI_BagButtonCapsule:SetWidth(220);
		XUI_BagButtonCapsuleArt:SetWidth(220);
		XUI_PetButtonCapsule:SetWidth(370);
		XUI_PetButtonCapsuleArt:SetWidth(370);
	elseif (XUI_ActionButtonGap == 5) then
		XUI_BagButtonCapsule:SetWidth(216);
		XUI_BagButtonCapsuleArt:SetWidth(216);
		XUI_PetButtonCapsule:SetWidth(361);
		XUI_PetButtonCapsuleArt:SetWidth(361);
	elseif (XUI_ActionButtonGap == 4) then
		XUI_BagButtonCapsule:SetWidth(212);
		XUI_BagButtonCapsuleArt:SetWidth(212);
		XUI_PetButtonCapsule:SetWidth(352);
		XUI_PetButtonCapsuleArt:SetWidth(352);
	elseif (XUI_ActionButtonGap == 3) then
		XUI_BagButtonCapsule:SetWidth(208);
		XUI_BagButtonCapsuleArt:SetWidth(208);
		XUI_PetButtonCapsule:SetWidth(343);
		XUI_PetButtonCapsuleArt:SetWidth(343);
	elseif (XUI_ActionButtonGap == 2) then
		XUI_BagButtonCapsule:SetWidth(204);
		XUI_BagButtonCapsuleArt:SetWidth(204);
		XUI_PetButtonCapsule:SetWidth(334);
		XUI_PetButtonCapsuleArt:SetWidth(334);
	end	
	if (XUI_EnableHotBar == 1) then
		if (XUI_ShowBagButtons == 1) then
			XUI_UpdateBagButtons();
		end
		if (XUI_ShowClassButtons == 1) then
			XUI_UpdateClassButtons();
		end
		if (XUI_ShowPetButtons == 1) then
			XUI_UpdatePetButtons();
		end
	end
end

-- ** SLASH COMMAND HANDLER FUNCTIONS ** --
function XUI_ActionButtonGapSlashHandler(msg)
	msg = string.lower(msg);
	if (msg == "default" or msg == "reset" or msg == "revert") then
		XUI_ActionButtonGap = XUI_DefaultActionButtonGap;
		XUI_ActionButtonDisplayUpdate();
		DEFAULT_CHAT_FRAME:AddMessage("Reverting action button gap to default amount.", 1, 1, 1);
	elseif (msg == "help") then
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /hotbarbuttongap /hbbuttongap", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   8, 6, 4, 2 - Set gap between action buttons to this value.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default state.", 1, 1, 1);
	elseif (msg ~= "8" and msg ~= "6" and msg ~= "4" and msg ~= "2") then
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /hotbarbuttongap /hbbuttongap", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   8, 6, 4, 2 - Set gap between action buttons to this value.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default state.", 1, 1, 1);
	else
		XUI_ActionButtonGap = msg;
		XUI_ActionButtonDisplayUpdate();
		DEFAULT_CHAT_FRAME:AddMessage("Setting action button gap to "..msg.." pixels.", 1, 1, 1);
	end
end

function XUI_LockHotBarSlashHandler(msg)
	msg = string.lower(msg);
	if (msg == "yes" or msg == "1" or msg == "true") then
		XUI_LockActionBar = 1;
		DEFAULT_CHAT_FRAME:AddMessage("Locking HotBar capsule.", 1, 1, 1);
	elseif (msg == "no" or msg == "0" or msg == "false") then
		XUI_LockActionBar = 0;
		DEFAULT_CHAT_FRAME:AddMessage("Unlocking HotBar capsule.", 1, 1, 1);
	elseif (msg == "default" or msg == "reset" or msg == "revert") then
		XUI_LockActionBar = XUI_DefaultLockActionBar;
		DEFAULT_CHAT_FRAME:AddMessage("Reverting HotBar lock state to default.", 1, 1, 1);
	elseif (msg == "help") then
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /hotbarlock /hblock", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   yes, true, or 1 - Locks HotBar capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   no, false, or 0 - Unlocks HotBar capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default state.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   blank - Toggle based on current state.", 1, 1, 1);
	elseif (msg == "") then
		if (XUI_LockActionBar == 1) then
			XUI_LockActionBar = 0;
			DEFAULT_CHAT_FRAME:AddMessage("Unlocking HotBar capsule.", 1, 1, 1);
		else
			XUI_LockActionBar = 1;
			DEFAULT_CHAT_FRAME:AddMessage("Locking HotBar capsule.", 1, 1, 1);
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /hotbarlock /hblock", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   yes, true, or 1 - Locks HotBar capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   no, false, or 0 - Unlocks HotBar capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default state.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   blank - Toggle based on current state.", 1, 1, 1);
	end
	XUI_UpdateModKeys();
end

function XUI_LockCastCapsuleSlashHandler(msg)
	msg = string.lower(msg);
	if (msg == "yes" or msg == "1" or msg == "true") then
		XUI_LockCastCapsule = 1;
		DEFAULT_CHAT_FRAME:AddMessage("Locking casting bar capsule.", 1, 1, 1);
	elseif (msg == "no" or msg == "0" or msg == "false") then
		XUI_LockCastCapsule = 0;
		DEFAULT_CHAT_FRAME:AddMessage("Unlocking casting bar capsule", 1, 1, 1);
	elseif (msg == "default" or msg == "reset" or msg == "revert") then
		XUI_LockCastCapsule = XUI_DefaultLockCastCapsule;
		DEFAULT_CHAT_FRAME:AddMessage("Reverting casting bar capsule lock state to default.", 1, 1, 1);
	elseif (msg == "help") then
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /generallockcastcapsule /genlockcastcapsule", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   yes, true, or 1 - Lock casting bar capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   no, false, or 0 - Unlock casting bar capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default state.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   blank - Toggle based on current state.", 1, 1, 1);
	elseif (msg == "") then
		if (XUI_LockCastCapsule == 1) then
			XUI_LockCastCapsule = 0;
			DEFAULT_CHAT_FRAME:AddMessage("Unlocking casting bar capsule.", 1, 1, 1);
		else
			XUI_LockCastCapsule = 1;
			DEFAULT_CHAT_FRAME:AddMessage("Locking casting bar capsule.", 1, 1, 1);
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /generallockcastcapsule /genlockcastcapsule", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   yes, true, or 1 - Lock casting bar capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   no, false, or 0 - Unlock casting bar capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default state.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   blank - Toggle based on current state.", 1, 1, 1);
	end	
end

function XUI_ResetCastCapsuleSlashHandler(msg)
	XUI_ResetCastBarCapsule();
	DEFAULT_CHAT_FRAME:AddMessage("Resetting casting bar capsule.", 1, 1, 1);
end

function XUI_GeneralSlashHandler(msg)
	DEFAULT_CHAT_FRAME:AddMessage("Valid slash commands for the General add-on", 1, 0.89, 0.01);
	DEFAULT_CHAT_FRAME:AddMessage("All commands may begin with either /general or /gen, ex. /generalquestfade", 1, 0.80, 0.01);
	DEFAULT_CHAT_FRAME:AddMessage("Entering help after any command will display a list of valid parameters.", 1, 0.80, 0);
	DEFAULT_CHAT_FRAME:AddMessage("   enablecastcapsule - Enable/Disable casting bar capsule.", 1, 1, 1);
	DEFAULT_CHAT_FRAME:AddMessage("   lockcastcapsule - Lock/Unlock the casting bar capsule.", 1, 1, 1);
	DEFAULT_CHAT_FRAME:AddMessage("   resetcastcapsule - Reset the casting bar capsule.", 1, 1, 1);
	DEFAULT_CHAT_FRAME:AddMessage("   questfade - Enable/Disable quest text fading.", 1, 1, 1);
	DEFAULT_CHAT_FRAME:AddMessage("   showquestlevels - Show/Hide quest levels in the quest log.", 1, 1, 1);
	DEFAULT_CHAT_FRAME:AddMessage("   tooltiptopmiddle - Position the tooltip at the top middle of the screen.", 1, 1, 1);
	DEFAULT_CHAT_FRAME:AddMessage("   tooltipcursor - Position the tooltip to follow the cursor.", 1, 1, 1);
	DEFAULT_CHAT_FRAME:AddMessage("   lagmeter - Toggle the XUIMod lag meter.", 1, 1, 1);	
end	

function XUI_AnchorCastingFrameSlashHandler(msg)
	-- Due to the nature of slash commands, 'msg' will never be nil, so we don't require a check
	msg = string.lower(msg);
	if (msg == "enable" or msg == "1" or msg == "true") then
		XUI_AnchorCastingFrame = 1;
		DEFAULT_CHAT_FRAME:AddMessage("Enabling cast bar capsule.", 1, 1, 1);
	elseif (msg == "disable" or msg == "0" or msg == "false") then
		XUI_AnchorCastingFrame = 0;
		DEFAULT_CHAT_FRAME:AddMessage("Disabling cast bar capsule.", 1, 1, 1);
	elseif (msg == "default" or msg == "reset" or msg == "revert") then
		XUI_AnchorCastingFrame = XUI_DefaultAnchorCastingFrame;
		DEFAULT_CHAT_FRAME:AddMessage("Reverting cast bar capsule state to default.", 1, 1, 1);
	elseif (msg == "help") then
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /generalenablecastcapsule /genenablecastcapsule", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   enable, true, or 1 - Enable casting bar capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   disable, false, or 0 - Disable casting bar capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default state.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   blank - Toggle based on current state.", 1, 1, 1);
	elseif (msg == "") then
		if (XUI_AnchorCastingFrame == 1) then 
			XUI_AnchorCastingFrame = 0; 
			DEFAULT_CHAT_FRAME:AddMessage("Disabling cast bar capsule.", 1, 1, 1);
		else 
			XUI_AnchorCastingFrame = 1; 
			DEFAULT_CHAT_FRAME:AddMessage("Enabling cast bar capsule.", 1, 1, 1);
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /generalenablecastcapsule /genenablecastcapsule", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   enable, true, or 1 - Enable casting bar capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   disable, false, or 0 - Disable casting bar capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default state.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   blank - Toggle based on current state.", 1, 1, 1);	
	end
	XUI_UpdateCastFrameAnchor();
end

function XUI_ShowLagMeterSlashHandler(msg)
	-- Due to the nature of slash commands, 'msg' will never be nil, so we don't require a check
	msg = string.lower(msg);
	if (msg == "show" or msg == "1" or msg == "true") then
		XUI_ShowLagMeter = 1;
		DEFAULT_CHAT_FRAME:AddMessage("Showing XUIMod lag meter.", 1, 1, 1);
	elseif (msg == "hide" or msg == "0" or msg == "false") then
		XUI_ShowLagMeter = 0;
		DEFAULT_CHAT_FRAME:AddMessage("Hiding XUIMod lag meter.", 1, 1, 1);
	elseif (msg == "default" or msg == "reset" or msg == "revert") then
		XUI_ShowLagMeter = XUI_DefaultShowLagMeter;
		DEFAULT_CHAT_FRAME:AddMessage("Reverting lag meter state to default.", 1, 1, 1);
	elseif (msg == "help") then
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /generallagmeter /genlagmeter", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   show, true, or 1 - Show XUIMod lag meter.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   hide, false, or 0 - Hide XUIMod lag meter.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default state.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   blank - Toggle based on current state.", 1, 1, 1);
	elseif (msg == "") then
		if (XUI_ShowLagMeter == 1) then 
			XUI_ShowLagMeter = 0; 
			DEFAULT_CHAT_FRAME:AddMessage("Hiding XUIMod lag meter.", 1, 1, 1);
		else 
			XUI_ShowLagMeter = 1; 
			DEFAULT_CHAT_FRAME:AddMessage("Showing XUIMod lag meter.", 1, 1, 1);
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /generallagmeter /genlagmeter", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   show, true, or 1 - Show XUIMod lag meter.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   hide, false, or 0 - Hide XUIMod lag meter.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default state.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   blank - Toggle based on current state.", 1, 1, 1);	
	end
	XUI_ToggleLagMeter();
end

function XUI_HotBarResetSlashHandler(msg)
	XUI_ActionBarReset();
	DEFAULT_CHAT_FRAME:AddMessage("Resetting all HotBar parameters.", 1, 1, 1);
end

function XUI_EnableHotBarSlashHandler(msg)
	-- Due to the nature of slash commands, 'msg' will never be nil, so we don't require a check
	msg = string.lower(msg);
	if (msg == "enable" or msg == "1" or msg == "true") then
		XUI_EnableHotBar = 1;
		DEFAULT_CHAT_FRAME:AddMessage("Enabling HotBar.", 1, 1, 1);
	elseif (msg == "disable" or msg == "0" or msg == "false") then
		XUI_EnableHotBar= 0;
		DEFAULT_CHAT_FRAME:AddMessage("Disabling HotBar.", 1, 1, 1);
	elseif (msg == "default" or msg == "reset" or msg == "revert") then
		XUI_EnableHotBar = XUI_DefaultEnableHotBar;
		DEFAULT_CHAT_FRAME:AddMessage("Reverting hot bar enable option to default.", 1, 1, 1);
	elseif (msg == "help") then
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /hotbarenable /hbenable", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   enable, true, or 1 - Enable HotBar.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   disable, false, or 0 - Disable HotBar, showing the default main bar.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default state.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   blank - Toggle based on current state.", 1, 1, 1);
	elseif (msg == "") then
		if (XUI_EnableHotBar == 1) then 
			XUI_EnableHotBar = 0; 
			DEFAULT_CHAT_FRAME:AddMessage("Disabling HotBar.", 1, 1, 1);
		else 
			XUI_EnableHotBar = 1; 
			DEFAULT_CHAT_FRAME:AddMessage("Enabling HotBar.", 1, 1, 1);
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /hotbarenable /hbenable", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   enable, true, or 1 - Enable HotBar.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   disable, false, or 0 - Disable HotBar, showing the default main bar.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default state.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   blank - Toggle based on current state.", 1, 1, 1);
	end
	XUI_UpdateHotBar();
end

function XUI_HotBarSlashHandler(msg)
	DEFAULT_CHAT_FRAME:AddMessage("Valid slash commands for the HotBar add-on", 1, 0.89, 0.01);
	DEFAULT_CHAT_FRAME:AddMessage("All commands may begin with either /hotbar or /hb, ex. /hotbarplaceholders", 1, 0.80, 0.01);
	DEFAULT_CHAT_FRAME:AddMessage("Entering help after any command will display a list of valid parameters.", 1, 0.80, 0);
	DEFAULT_CHAT_FRAME:AddMessage("   enable - Enable/Disable XUIMod HotBar.", 1, 1, 1);
	DEFAULT_CHAT_FRAME:AddMessage("   lock - Lock/Unlock HotBar capsule.", 1, 1, 1);
	DEFAULT_CHAT_FRAME:AddMessage("   buttongap - Set gap between action buttons.", 1, 1, 1);
	DEFAULT_CHAT_FRAME:AddMessage("   showmenubuttons - Show/Hide menu buttons with capsule.", 1, 1, 1);
	DEFAULT_CHAT_FRAME:AddMessage("   lockmenubuttons - Lock/Unlock menu button capsule.", 1, 1, 1);
	DEFAULT_CHAT_FRAME:AddMessage("   showbagbuttons - Show/Hide menu buttons with capsule.", 1, 1, 1);
	DEFAULT_CHAT_FRAME:AddMessage("   lockbagbuttons - Lock/Unlock menu button capsule.", 1, 1, 1);
	DEFAULT_CHAT_FRAME:AddMessage("   showpetbuttons - Show/Hide menu buttons with capsule.", 1, 1, 1);
	DEFAULT_CHAT_FRAME:AddMessage("   lockpetbuttons - Lock/Unlock menu button capsule.", 1, 1, 1);
	DEFAULT_CHAT_FRAME:AddMessage("   showclassbuttons - Show/Hide menu buttons with capsule.", 1, 1, 1);
	DEFAULT_CHAT_FRAME:AddMessage("   lockclassbuttons - Lock/Unlock menu button capsule.", 1, 1, 1);
end
