--[[
	XUI_MicroButtons.lua
	XUIVersion++2004.11.15++

	Capsule to allow movement of micro menu buttons.
	
]]

-- ** DEFAULT SETTINGS **--
-- Default MicroButton capsule show, off
XUI_DefaultShowMicroButtons = 0;
-- Default local lock setting
XUI_DefaultLockMicroButtonsCapsule = 0;

-- ** GENERAL VARIABLES ** --
-- Save original talent button update function
XUI_OriginalUpdateTalentButton = UpdateTalentButton;
-- Show menu buttons bunding name
BINDING_NAME_XUISHOWMENUBUTTONS = "Toggle Menu Buttons";

-- ** MICRO BUTTON CAPSULE FUNCTIONS ** --
-- Event registers
function XUI_MicroButtonCapsuleOnLoad ()
	this:RegisterEvent("VARIABLES_LOADED");
end

-- Registrations
function XUI_MicroButtonCapsuleOnEvent (event)
	if (event == "VARIABLES_LOADED") then
	XUI_UpdateActionButtonGap();
		if (XUI_SHELL == 1) then
			if (XUI_RetrieveSaved("ShowMicroButtons") == nil) then
				XUI_ShowMicroButtons = XUI_DefaultShowMicroButtons;
			else
				XUI_ShowMicroButtons = XUI_RetrieveSaved("ShowMicroButtons");
			end
			XUI_RegisterOption(120, "check", XUI_ShowMicroButtons, "ShowMicroButtons", XUI_UpdateMicroButtons, "Show Menu Buttons", "Shows the default action bar menu buttons\nin their own movable capsule.\nNote this option can be duplicated with a key\nbinding to quickly show or hide the buttons.");
		else
			if (XUI_ShowMicroButtons == nil) then
				XUI_ShowMicroButtons = XUI_DefaultShowMicroButtons;
			end
			--RegisterForSave("XUI_ShowMicroButtons");
			SlashCmdList["XUI_SHOWMICROBUTTONS"] = XUI_ShowMicroButtonsSlashHandler;
			SLASH_XUI_SHOWMICROBUTTONS1 = "/hotbarshowmenubuttons";
			SLASH_XUI_SHOWMICROBUTTONS2 = "/hbshowmenubuttons";
			SlashCmdList["XUI_LOCKMICROBUTTONS"] = XUI_LockMicroButtonsSlashHandler;
			SLASH_XUI_LOCKMICROBUTTONS1 = "/hotbarlockmenubuttons";
			SLASH_XUI_LOCKMICROBUTTONS2 = "/hblockmenubuttons";
		end
		return;
	end
end	

-- Set the capsule backdrop if it shows
function XUI_MicroButtonCapsuleArtOnShow ()
	this:SetBackdropBorderColor(0, 0, 0);
	this:SetBackdropColor(0, 0, 0);
end

-- Function to show and hide the menu when the key binding to do so is pressed
function XUI_ShowMenuExecuteBinding ()
	if (XUI_RetrieveOption ~= nil) then
		if (XUI_RetrieveOption(120) ~= nil) then
			XUI_ShowMicroButtons = XUI_RetrieveOption(120)[XUI_VALUE];
		end
	end
	-- Simply toggle
	if (XUI_ShowMicroButtons == 1) then
		XUI_ShowMicroButtons = 0;
	else
		XUI_ShowMicroButtons = 1;
	end
	-- Update setting in Shell
	if (XUI_SHELL == 1) then
		XUI_UpdateValue(120, XUI_ShowMicroButtons);
	end
	-- Update function
	XUI_UpdateMicroButtons();
end

-- Capsule watch
function XUI_MicroButtonCapsuleOnUpdate ()
	XUI_ToggleMicroButtonCapsule();
end

-- Capsule toggle
function XUI_ToggleMicroButtonCapsule ()
	if (XUI_RetrieveOption ~= nil) then
		if (XUI_RetrieveOption(120) ~= nil) then
			XUI_ShowMicroButtons = XUI_RetrieveOption(120)[XUI_VALUE];
		end
	end
	if (XUI_ShowMicroButtons == 1) then
		-- If Shell is present, update the local lock setting
		if (XUI_SHELL ~= nil) then
			XUI_LockMicroButtonsCapsule = XUI_RetrieveOption(102)[XUI_VALUE];
		end	
		-- Then, if the action bar is not locked, go ahead and show/hide or drag the capsule as needed, or if it is locked, hide the capsule so it can't be dragged
		if (XUI_LockMicroButtonsCapsule == 0) then
			-- Make sure the capsule is showing, it might have been previously hidden
			if (not XUI_MicroButtonCapsule:IsVisible()) then
				XUI_MicroButtonCapsule:Show();
			end
			-- Quick function to check if the mouse is anywhere over our capsule
			if (MouseIsOver(XUI_MicroButtonCapsule)) then
				XUI_MicroButtonCapsuleArt:Show();
			else
				XUI_MicroButtonCapsuleArt:Hide();
			end
		else
			XUI_MicroButtonCapsule:Hide();
		end
	else
		XUI_MicroButtonCapsule:Hide();
	end
end

-- Move/Show micro buttons as we need
function XUI_UpdateMicroButtons ()
	if (XUI_RetrieveOption ~= nil) then
		if (XUI_RetrieveOption(101) ~= nil) then
			XUI_EnableHotBar = XUI_RetrieveOption(101)[XUI_VALUE];
		end
		if (XUI_RetrieveOption(120) ~= nil) then
			XUI_ShowMicroButtons = XUI_RetrieveOption(120)[XUI_VALUE];
		end
	end
	-- If the HotBar is enabled, determine if we're showing micro buttons or not. Else show them in their default spots, and hide the capsule.
	if (XUI_EnableHotBar == 1) then
		-- Show micro buttons, reroute the talent button function, and anchor to the capsule
		if (XUI_ShowMicroButtons == 1) then
			XUI_MicroButtonCapsule:Show();
			CharacterMicroButton:Show();
			CharacterMicroButton:ClearAllPoints();
			CharacterMicroButton:SetPoint("BOTTOMLEFT", "XUI_MicroButtonCapsule", "BOTTOMLEFT", 6, 6);
			SpellbookMicroButton:Show();
			QuestLogMicroButton:Show();
			SocialsMicroButton:Show();
			WorldMapMicroButton:Show();
			MainMenuMicroButton:Show();
			HelpMicroButton:Show();
			UpdateTalentButton = XUI_OriginalUpdateTalentButton;
			UpdateTalentButton();
		else
			-- Hide everything, destroy the pesky talent button function
			XUI_MicroButtonCapsule:Hide();
			CharacterMicroButton:Hide();
			SpellbookMicroButton:Hide();
			QuestLogMicroButton:Hide();
			SocialsMicroButton:Hide();
			WorldMapMicroButton:Hide();
			MainMenuMicroButton:Hide();
			HelpMicroButton:Hide();	
			TalentMicroButton:Hide();
			UpdateTalentButton = function()
				return;
			end
		end
	else
		-- Hide the capsule, reset buttons to default position, restore talent button function
		XUI_MicroButtonCapsule:Hide();
		CharacterMicroButton:Show();
		CharacterMicroButton:ClearAllPoints();
		CharacterMicroButton:SetPoint("BOTTOMLEFT", "MainMenuBarArtFrame", "BOTTOMLEFT", 546, 2);
		SpellbookMicroButton:Show();
		QuestLogMicroButton:Show();
		SocialsMicroButton:Show();
		WorldMapMicroButton:Show();
		MainMenuMicroButton:Show();
		HelpMicroButton:Show();
		UpdateTalentButton = XUI_OriginalUpdateTalentButton;
		UpdateTalentButton();
	end
end

-- ** SLASH HANDLERS ** --
	
function XUI_ShowMicroButtonsSlashHandler (msg)
	-- Due to the nature of slash commands, 'msg' will never be nil, so we don't require a check
	msg = string.lower(msg);
	if (msg == "show" or msg == "1" or msg == "true") then
		XUI_ShowMicroButtons = 1;
		DEFAULT_CHAT_FRAME:AddMessage("Showing menu buttons & capsule.", 1, 1, 1);
	elseif (msg == "hide" or msg == "0" or msg == "false") then
		XUI_ShowMicroButtons= 0;
		DEFAULT_CHAT_FRAME:AddMessage("Hiding menu buttons & capsule.", 1, 1, 1);
	elseif (msg == "default" or msg == "reset" or msg == "revert") then
		XUI_ShowMicroButtons = XUI_DefaultShowMicroButtons;
		DEFAULT_CHAT_FRAME:AddMessage("Reverting menu button display state to default.", 1, 1, 1);
	elseif (msg == "help") then
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /hotbarshowmenubuttons /hbshowmenubuttons", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   show, true, or 1 - Show menu buttons & capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   hide, false, or 0 - Hide menu buttons & capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default state.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   blank - Toggle based on current state.", 1, 1, 1);
	elseif (msg == "") then
		if (XUI_ShowMicroButtons == 1) then 
			XUI_ShowMicroButtons = 0; 
			DEFAULT_CHAT_FRAME:AddMessage("Hiding menu buttons & capsule.", 1, 1, 1);
		else 
			XUI_ShowMicroButtons = 1; 
			DEFAULT_CHAT_FRAME:AddMessage("Showing menu buttons & capsule.", 1, 1, 1);
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /hotbarshowmenubuttons /hbshowmenubuttons", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   show, true, or 1 - Show menu buttons & capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   hide, false, or 0 - Hide menu buttons & capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default state.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   blank - Toggle based on current state.", 1, 1, 1);
	end
	XUI_UpdateMicroButtons();
end

function XUI_LockMicroButtonsSlashHandler (msg)
	-- Due to the nature of slash commands, 'msg' will never be nil, so we don't require a check
	msg = string.lower(msg);
	if (msg == "yes" or msg == "1" or msg == "true") then
		XUI_LockMicroButtonsCapsule = 1;
		DEFAULT_CHAT_FRAME:AddMessage("Locking menu buttons capsule.", 1, 1, 1);
	elseif (msg == "no" or msg == "0" or msg == "false") then
		XUI_LockMicroButtonsCapsule= 0;
		DEFAULT_CHAT_FRAME:AddMessage("Unlocking menu buttons capsule.", 1, 1, 1);
	elseif (msg == "default" or msg == "reset" or msg == "revert") then
		XUI_LockMicroButtonsCapsule = XUI_DefaultLockMicroButtonsCapsule;
		DEFAULT_CHAT_FRAME:AddMessage("Reverting menu button capsule lock state to default.", 1, 1, 1);
	elseif (msg == "help") then
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /hotbarlockmenubuttons /hblockmenubuttons", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   yes, true, or 1 - Lock menu buttons capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   no, false, or 0 - Unlock menu buttons capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default state.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   blank - Toggle based on current state.", 1, 1, 1);
	elseif (msg == "") then
		if (XUI_LockMicroButtonsCapsule == 1) then 
			XUI_LockMicroButtonsCapsule = 0; 
			DEFAULT_CHAT_FRAME:AddMessage("Unlocking menu buttons capsule.", 1, 1, 1);
		else 
			XUI_LockMicroButtonsCapsule = 1; 
			DEFAULT_CHAT_FRAME:AddMessage("Locking menu buttons capsule.", 1, 1, 1);
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /hotbarlockmenubuttons /hblockmenubuttons", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   yes, true, or 1 - Lock menu buttons capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   no, false, or 0 - Unlock menu buttons capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default state.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   blank - Toggle based on current state.", 1, 1, 1);
	end
end
