--[[
	XUI_PetButtons.lua
	XUIVersion++2004.11.15++

	Capsule to allow movement of Pet buttons.
	
]]

-- ** DEFAULT SETTINGS **--
-- Default PetButton capsule show, off
XUI_DefaultShowPetButtons = 1;
-- Default local lock setting
XUI_DefaultLockPetButtonsCapsule = 0;

-- ** GENERAL VARIABLES ** --
-- Show Pet buttons binding name
BINDING_NAME_XUISHOWPETBUTTONS = "Toggle Pet Buttons";

-- ** Pet BUTTON CAPSULE FUNCTIONS ** --
-- Event registers
function XUI_PetButtonCapsuleOnLoad ()
	this:RegisterEvent("VARIABLES_LOADED");
end

-- Registrations
function XUI_PetButtonCapsuleOnEvent (event)
	if (event == "VARIABLES_LOADED") then
		if (XUI_SHELL == 1) then
			if (XUI_RetrieveSaved("ShowPetButtons") == nil) then
				XUI_ShowPetButtons = XUI_DefaultShowPetButtons;
			else
				XUI_ShowPetButtons = XUI_RetrieveSaved("ShowPetButtons");
			end
			XUI_RegisterOption(122, "check", XUI_ShowPetButtons, "ShowPetButtons", XUI_UpdatePetButtons, "Show Pet Buttons", "Shows the default action bar Pet buttons\nin their own movable capsule.\nNote this option can be duplicated with a key\nbinding to quickly show or hide the buttons.");
		else
			if (XUI_ShowPetButtons == nil) then
				XUI_ShowPetButtons = XUI_DefaultShowPetButtons;
			end
			--RegisterForSave("XUI_ShowPetButtons");
			SlashCmdList["XUI_SHOWPetBUTTONS"] = XUI_ShowPetButtonsSlashHandler;
			SLASH_XUI_SHOWPetBUTTONS1 = "/hotbarshowPetbuttons";
			SLASH_XUI_SHOWPetBUTTONS2 = "/hbshowPetbuttons";
			SlashCmdList["XUI_LOCKPetBUTTONS"] = XUI_LockPetButtonsSlashHandler;
			SLASH_XUI_LOCKPetBUTTONS1 = "/hotbarlockPetbuttons";
			SLASH_XUI_LOCKPetBUTTONS2 = "/hblockPetbuttons";
		end
		return;
	end
end	

-- Set the capsule backdrop if it shows
function XUI_PetButtonCapsuleArtOnShow ()
	this:SetBackdropBorderColor(0, 0, 0);
	this:SetBackdropColor(0, 0, 0);
end

-- Function to show and hide the Pet buttons when the key binding to do so is pressed
function XUI_ShowPetExecuteBinding ()
	if (XUI_RetrieveOption ~= nil) then
		if (XUI_RetrieveOption(122) ~= nil) then
			XUI_ShowPetButtons = XUI_RetrieveOption(122)[XUI_VALUE];
		end
	end
	-- Simply toggle
	if (XUI_ShowPetButtons == 1) then
		XUI_ShowPetButtons = 0;
	else
		XUI_ShowPetButtons = 1;
	end
	-- Update setting in Shell
	if (XUI_SHELL == 1) then
		XUI_UpdateValue(122, XUI_ShowPetButtons);
	end
	-- Update function
	XUI_UpdatePetButtons();
end

-- Capsule watch
function XUI_PetButtonCapsuleOnUpdate ()
	XUI_TogglePetButtonCapsule();
end

-- Capsule toggle
function XUI_TogglePetButtonCapsule ()
	if (XUI_RetrieveOption ~= nil) then
		if (XUI_RetrieveOption(122) ~= nil) then
			XUI_ShowPetButtons = XUI_RetrieveOption(122)[XUI_VALUE];
		end
	end
	if (XUI_ShowPetButtons == 1) then
		-- If Shell is present, update the local lock setting
		if (XUI_SHELL ~= nil) then
			XUI_LockPetButtonsCapsule = XUI_RetrieveOption(102)[XUI_VALUE];
		end	
		-- Then, if the action bar is not locked, go ahead and show/hide or drag the capsule as needed, or if it is locked, hide the capsule so it can't be dragged
		if (XUI_LockPetButtonsCapsule == 0) then
			-- Make sure the capsule is showing, it might have been previously hidden
			if (not XUI_PetButtonCapsule:IsVisible()) then
				XUI_PetButtonCapsule:Show();
			end
			-- Quick function to check if the mouse is anywhere over our capsule
			if (MouseIsOver(XUI_PetButtonCapsule)) then
				XUI_PetButtonCapsuleArt:Show();
			else
				XUI_PetButtonCapsuleArt:Hide();
			end
		else
			XUI_PetButtonCapsule:Hide();
		end
	else
		XUI_PetButtonCapsule:Hide();
	end
end

-- Move/Show Pet buttons as we need
function XUI_UpdatePetButtons ()
	if (XUI_RetrieveOption ~= nil) then
		if (XUI_RetrieveOption(101) ~= nil) then
			XUI_EnableHotBar = XUI_RetrieveOption(101)[XUI_VALUE];
		end
		if (XUI_RetrieveOption(122) ~= nil) then
			XUI_ShowPetButtons = XUI_RetrieveOption(122)[XUI_VALUE];
		end
	end
	-- See if we're using the pet action frame
	if (XUI_EnableHotBar == 1) then
		-- Change anchors on the pet action buttons
		for i=1, 10 do
			-- Get current button
			local button = getglobal("PetActionButton"..i);
			-- Decrement the ID
			local lastID = i - 1;
			-- Get the last button
			local lastButton = "PetActionButton"..lastID;
			-- Need to use completely different gapping for when the HotBar is disabled
			if (XUI_ShowPetButtons == 1) then
				if (i == 1) then
					XUI_PetButtonCapsule:Show();
					local gap = XUI_ActionButtonGapMax - XUI_ActionButtonGap;
					local negativeGap = gap - (gap * 2)
					local useGap = negativeGap - 10;
					button:ClearAllPoints();
					button:SetPoint("TOPLEFT", "XUI_PetButtonCapsule", "TOPLEFT", 8, -22);
				else
					button:ClearAllPoints();
					button:SetPoint("LEFT", lastButton, "RIGHT", XUI_ActionButtonGap, 0);
				end
			else
				if (i == 1) then
					XUI_PetButtonCapsule:Hide();
					button:ClearAllPoints();
					button:SetPoint("BOTTOMLEFT", "PetActionBarFrame", "BOTTOMLEFT", 36, 1);
				else
					button:ClearAllPoints();
					button:SetPoint("LEFT", lastButton, "RIGHT", 8, 0);
				end
			end
		end
	end
end

-- ** SLASH HANDLERS ** --
function XUI_ShowPetButtonsSlashHandler (msg)
	-- Due to the nature of slash commands, 'msg' will never be nil, so we don't require a check
	msg = string.lower(msg);
	if (msg == "show" or msg == "1" or msg == "true") then
		XUI_ShowPetButtons = 1;
		DEFAULT_CHAT_FRAME:AddMessage("Showing Pet buttons & capsule.", 1, 1, 1);
	elseif (msg == "hide" or msg == "0" or msg == "false") then
		XUI_ShowPetButtons= 0;
		DEFAULT_CHAT_FRAME:AddMessage("Hiding Pet buttons & capsule.", 1, 1, 1);
	elseif (msg == "default" or msg == "reset" or msg == "revert") then
		XUI_ShowPetButtons = XUI_DefaultShowPetButtons;
		DEFAULT_CHAT_FRAME:AddMessage("Reverting Pet button display state to default.", 1, 1, 1);
	elseif (msg == "help") then
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /hotbarshowPetbuttons /hbshowPetbuttons", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   show, true, or 1 - Show Pet buttons & capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   hide, false, or 0 - Hide Pet buttons & capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default state.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   blank - Toggle based on current state.", 1, 1, 1);
	elseif (msg == "") then
		if (XUI_ShowPetButtons == 1) then 
			XUI_ShowPetButtons = 0; 
			DEFAULT_CHAT_FRAME:AddMessage("Hiding Pet buttons & capsule.", 1, 1, 1);
		else 
			XUI_ShowPetButtons = 1; 
			DEFAULT_CHAT_FRAME:AddMessage("Showing Pet buttons & capsule.", 1, 1, 1);
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /hotbarshowPetbuttons /hbshowPetbuttons", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   show, true, or 1 - Show Pet buttons & capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   hide, false, or 0 - Hide Pet buttons & capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default state.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   blank - Toggle based on current state.", 1, 1, 1);
	end
	XUI_UpdatePetButtons();
end

function XUI_LockPetButtonsSlashHandler (msg)
	-- Due to the nature of slash commands, 'msg' will never be nil, so we don't require a check
	msg = string.lower(msg);
	if (msg == "yes" or msg == "1" or msg == "true") then
		XUI_LockPetButtonsCapsule = 1;
		DEFAULT_CHAT_FRAME:AddMessage("Locking Pet buttons capsule.", 1, 1, 1);
	elseif (msg == "no" or msg == "0" or msg == "false") then
		XUI_LockPetButtonsCapsule= 0;
		DEFAULT_CHAT_FRAME:AddMessage("Unlocking Pet buttons capsule.", 1, 1, 1);
	elseif (msg == "default" or msg == "reset" or msg == "revert") then
		XUI_LockPetButtonsCapsule = XUI_DefaultLockPetButtonsCapsule;
		DEFAULT_CHAT_FRAME:AddMessage("Reverting Pet button capsule lock state to default.", 1, 1, 1);
	elseif (msg == "help") then
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /hotbarlockPetbuttons /hblockPetbuttons", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   yes, true, or 1 - Lock Pet buttons capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   no, false, or 0 - Unlock Pet buttons capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default state.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   blank - Toggle based on current state.", 1, 1, 1);
	elseif (msg == "") then
		if (XUI_LockPetButtonsCapsule == 1) then 
			XUI_LockPetButtonsCapsule = 0; 
			DEFAULT_CHAT_FRAME:AddMessage("Unlocking Pet buttons capsule.", 1, 1, 1);
		else 
			XUI_LockPetButtonsCapsule = 1; 
			DEFAULT_CHAT_FRAME:AddMessage("Locking Pet buttons capsule.", 1, 1, 1);
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /hotbarlockPetbuttons /hblockPetbuttons", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   yes, true, or 1 - Lock Pet buttons capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   no, false, or 0 - Unlock Pet buttons capsule.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default state.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   blank - Toggle based on current state.", 1, 1, 1);
	end
end
