--[[
	XUI_GlobalFunctions.lua
	XUIVersion++2004.11.07++		
	
	Functions useful to any file, and debug functions.
]]

-- Print out a message prefixed by XUIMsg to the default chat frame
function XUI_Print(msg, r, g, b)
	if(r == nil) then r = 1; end
	if(g == nil) then g = 1; end
	if(b == nil) then b = 1; end
	DEFAULT_CHAT_FRAME:AddMessage("XUIMsg: " .. msg, r, g, b);
end

-- Printer for XUIMod errors that don't need to break the interface or reference code
function XUI_Error(msg)
	DEFAULT_CHAT_FRAME:AddMessage("XUIMod: "..msg, 1, 0.25, 0.25);
end

-- ** DEBUG FUNCTIONS ** --

-- Debug-specific printer
function XUI_DebugPrint (msg, r, g, b)
	if(r == nil) then r = 1; end
	if(g == nil) then g = 1; end
	if(b == nil) then b = 1; end
	DEFAULT_CHAT_FRAME:AddMessage(">]"..msg, r, g, b);
end

-- Print our entire registration table
function XUI_PrintAllRegistrations()
	local n = getn(XUI_REGISTRATIONS);
	for i=1, n do
		local id = XUI_REGISTRATIONS[i][XUI_ID];
		XUI_PrintRegistration(id);
	end
end

-- Print a registration at index 'i'
function XUI_PrintRegistration(i)
	if (XUI_RetrieveOption(i) ~= nil) then
		local id = XUI_RetrieveOption(i)[XUI_ID];
		local type = XUI_RetrieveOption(i)[XUI_TYPE];
		local value = XUI_RetrieveOption(i)[XUI_VALUE];
		local name = XUI_RetrieveOption(i)[XUI_NAME];
		local func = XUI_RetrieveOption(i)[XUI_FUNC];
		local label = XUI_RetrieveOption(i)[XUI_LABEL];
		local tooltip = XUI_RetrieveOption(i)[XUI_TOOLTIP];
		local slidermin = XUI_RetrieveOption(i)[XUI_SLIDMIN];
		local slidermax = XUI_RetrieveOption(i)[XUI_SLIDMAX];
		local sliderstep = XUI_RetrieveOption(i)[XUI_SLIDSTEP];
		local sliderunit = XUI_RetrieveOption(i)[XUI_SLIDUNIT];
		
									XUI_DebugPrint("XUI_REGISTRATIONS[".. i.."] = {");
									XUI_DebugPrint("   ["..XUI_ID.."] = "..id..",");
									XUI_DebugPrint("   ["..XUI_TYPE.."] = "..type..",");
		if (value ~= nil) then		XUI_DebugPrint("   ["..XUI_VALUE.."] = "..value..",");				end
		if (name ~= nil) then		XUI_DebugPrint("   ["..XUI_NAME.."] = "..name..",");				end
		if (func ~= nil) then		XUI_DebugPrint("   ["..XUI_FUNC.."] = NOT NIL,");					end
									XUI_DebugPrint("   ["..XUI_LABEL.."] = "..label..",");
		if (tooltip ~= nil) then	XUI_DebugPrint("   ["..XUI_TOOLTIP.."] = "..tooltip..",");			end
		if (slidermin ~= nil) then	XUI_DebugPrint("   ["..XUI_SLIDMIN.."] = "..slidermin..",");		end
		if (slidermax ~= nil) then	XUI_DebugPrint("   ["..XUI_SLIDMAX.."] = "..slidermax..",");		end
		if (sliderstep ~= nil) then	XUI_DebugPrint("   ["..XUI_SLIDSTEP.."] = "..sliderstep..",");		end
		if (sliderunit ~= nil) then	XUI_DebugPrint("   ["..XUI_SLIDUNIT.."] = "..sliderunit);			end
									XUI_DebugPrint("}");
	else
		XUI_DebugPrint('Nil argument');
	end
end

-- Print our entire saves array
function XUI_PrintAllSaves()
	XUI_DebugPrint("XUI_SAVED = {");
	local n = getn(XUI_REGISTRATIONS);
	for i=1, n do
		local id = XUI_REGISTRATIONS[i][XUI_ID];
		if (XUI_RetrieveOption(id)[XUI_NAME] ~= nil) then
			local name = XUI_RetrieveOption(id)[XUI_NAME];
			XUI_PrintSaved(name);
		end
	end
	XUI_DebugPrint("}");
end			

-- Print a saved value at 'name'
function XUI_PrintSaved(name)
	local value = XUI_SAVED[name];
	XUI_DebugPrint("   ["..name.."] = "..value..",");
end	

-- Function to print out memory used
function XUI_MemoryUsage()
	local memory = gcinfo();
	XUI_DebugPrint(memory);
end